@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020,2021 Willem Vermin
+#-# 	      2019,2020,2021,2022 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -34,15 +34,19 @@
 
 /*         code           type               name              modifier */
 #define ALL_TOGGLES  \
+   BUTTON(togglecode     ,xsnow_celestials  ,Aurora               ,1  )         \
+   BUTTON(togglecode     ,xsnow_celestials  ,AuroraLeft           ,1  )         \
+   BUTTON(togglecode     ,xsnow_celestials  ,AuroraMiddle         ,1  )         \
+   BUTTON(togglecode     ,xsnow_celestials  ,AuroraRight          ,1  )         \
    BUTTON(togglecode     ,xsnow_birds       ,BirdsOnly            ,1  )         \
    BUTTON(togglecode     ,xsnow_birds       ,FollowSanta          ,1  )         \
    BUTTON(togglecode     ,xsnow_birds       ,ShowAttrPoint        ,1  )         \
    BUTTON(togglecode     ,xsnow_birds       ,ShowBirds            ,1  )         \
    BUTTON(togglecode     ,xsnow_settings    ,AllWorkspaces        ,1  )         \
    BUTTON(togglecode     ,xsnow_settings    ,BelowAll             ,1  )         \
-   BUTTON(togglecode     ,xsnow_settings    ,BelowConfirm         ,0 )          \
+   BUTTON(togglecode     ,xsnow_settings    ,BelowConfirm         ,0  )         \
    BUTTON(togglecode     ,xsnow_settings    ,ThemeXsnow           ,1  )         \
-   BUTTON(togglecode     ,xsnow_celestials  ,NoMeteorites         ,-1 )   /*i*/ \
+   BUTTON(togglecode     ,xsnow_celestials  ,NoMeteors            ,-1 )   /*i*/ \
    BUTTON(togglecode     ,xsnow_celestials  ,Halo                 ,1  )         \
    BUTTON(togglecode     ,xsnow_celestials  ,Moon                 ,1  )         \
    BUTTON(togglecode     ,xsnow_santa       ,NoSanta              ,-1 )   /*i*/ \
@@ -54,8 +58,9 @@
    BUTTON(togglecode     ,xsnow_snow        ,NoSnowFlakes         ,-1 )   /*i*/ \
    BUTTON(togglecode     ,xsnow_celestials  ,Stars                ,1  )         \
    BUTTON(togglecode     ,xsnow_scenery     ,NoTrees              ,-1 )   /*i*/ \
-   BUTTON(togglecode     ,xsnow_scenery     ,Overlap              ,1 )          \
+   BUTTON(togglecode     ,xsnow_scenery     ,Overlap              ,1  )         \
    BUTTON(togglecode     ,xsnow_celestials  ,NoWind               ,-1 )   /*i*/ \
+   BUTTON(togglecode     ,xsnow_settings    ,BlackBackground      ,1  )         \
 
 
 #define ALL_SCALES \
@@ -69,12 +74,21 @@
    BUTTON(scalecode      ,xsnow_birds       ,Neighbours           ,1  )         \
    BUTTON(scalecode      ,xsnow_birds       ,PrefDistance         ,1  )         \
    BUTTON(scalecode      ,xsnow_birds       ,ViewingDistance      ,1  )         \
+   BUTTON(scalecode      ,xsnow_birds       ,AttrSpace            ,1  )         \
    BUTTON(scalecode      ,xsnow_settings    ,CpuLoad              ,1  )         \
    BUTTON(scalecode      ,xsnow_settings    ,OffsetS              ,-1 )   /*i*/ \
    BUTTON(scalecode      ,xsnow_settings    ,OffsetY              ,-1 )   /*i*/ \
    BUTTON(scalecode      ,xsnow_settings    ,Transparency         ,1  )         \
    BUTTON(scalecode      ,xsnow_settings    ,Scale                ,1  )         \
+   BUTTON(scalecode      ,xsnow_settings    ,IgnoreTop            ,1  )         \
+   BUTTON(scalecode      ,xsnow_settings    ,IgnoreBottom         ,1  )         \
+   BUTTON(scalecode      ,xsnow_celestials  ,AuroraWidth          ,1  )         \
+   BUTTON(scalecode      ,xsnow_celestials  ,AuroraBase           ,1  )         \
+   BUTTON(scalecode      ,xsnow_celestials  ,AuroraBrightness     ,1  )         \
+   BUTTON(scalecode      ,xsnow_celestials  ,AuroraHeight         ,1  )         \
+   BUTTON(scalecode      ,xsnow_celestials  ,AuroraSpeed          ,1  )         \
    BUTTON(scalecode      ,xsnow_celestials  ,HaloBright           ,1  )         \
+   BUTTON(scalecode      ,xsnow_celestials  ,MeteorFrequency      ,1  )         \
    BUTTON(scalecode      ,xsnow_celestials  ,MoonSize             ,1  )         \
    BUTTON(scalecode      ,xsnow_celestials  ,MoonSpeed            ,1  )         \
    BUTTON(scalecode      ,xsnow_santa       ,SantaSpeedFactor     ,1  )         \
@@ -98,11 +112,14 @@
    BUTTON(colorcode      ,xsnow_snow        ,SnowColor            ,1  )         \
    BUTTON(colorcode      ,xsnow_scenery     ,TreeColor            ,1  )         \
 
+#define ALL_FILECHOOSERS \
+   BUTTON(filecode       ,xsnow_settings    ,BackgroundFile       ,1  )
 
 #define BUTTON(code, type, name, m) code(type,name,m)
 
 #define ALL_BUTTONS \
    ALL_TOGGLES      \
    ALL_SCALES       \
-   ALL_COLORS
+   ALL_COLORS       \
+   ALL_FILECHOOSERS
 
