@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020,2021 Willem Vermin
+#-# 	      2019,2020,2021,2022 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 // are no part of the ui (except BelowAll)
 #define DOITALL \
    DOIT_I(BelowAll            ,1  ,1 ) \
+   DOIT_I(BelowAllForce       ,0  ,0 ) /* to know if the -above flag was used */\
    DOIT_I(BelowConfirm        ,0  ,0 ) /*not a button or parameter */ \
    DOIT_I(Changes             ,0  ,0 ) /* not a parameter or button */ \
    DOIT_I(CheckGtk            ,1  ,1 ) \
@@ -53,6 +54,15 @@
 // following flags are written to the config file and
 // are part of the ui
 #define DOIT  \
+   DOIT_I(Aurora               ,1          ,0          ) \
+   DOIT_I(AuroraBase           ,50         ,50         ) \
+   DOIT_I(AuroraBrightness     ,15         ,15         ) \
+   DOIT_I(AuroraHeight         ,30         ,30         ) \
+   DOIT_I(AuroraSpeed          ,50         ,50         ) \
+   DOIT_I(AuroraWidth          ,60         ,60         ) \
+   DOIT_I(AuroraLeft           ,0          ,0          ) \
+   DOIT_I(AuroraMiddle         ,1          ,1          ) \
+   DOIT_I(AuroraRight          ,0          ,0          ) \
    DOIT_I(AllWorkspaces        ,1          ,1          ) \
    DOIT_I(BlowOffFactor        ,40         ,40         ) \
    DOIT_I(BlowSnow             ,1          ,0          ) \
@@ -66,6 +76,7 @@
    DOIT_I(MaxOnTrees           ,200        ,200        ) \
    DOIT_I(MaxScrSnowDepth      ,50         ,50         ) \
    DOIT_I(MaxWinSnowDepth      ,30         ,30         ) \
+   DOIT_I(MeteorFrequency      ,40         ,40         ) \
    DOIT_I(Moon                 ,1          ,0          ) \
    DOIT_I(MoonSpeed            ,120        ,120        ) \
    DOIT_I(MoonSize             ,100        ,100        ) \
@@ -74,7 +85,7 @@
    DOIT_I(NoKeepSnow           ,0          ,0          ) \
    DOIT_I(NoKeepSnowOnTrees    ,0          ,1          ) \
    DOIT_I(NoKeepSWin           ,0          ,0          ) \
-   DOIT_I(NoMeteorites         ,0          ,1          ) \
+   DOIT_I(NoMeteors            ,0          ,1          ) \
    DOIT_I(NoSanta              ,0          ,0          ) \
    DOIT_I(NoSnowFlakes         ,0          ,0          ) \
    DOIT_I(NoTrees              ,0          ,0          ) \
@@ -97,9 +108,11 @@
    DOIT_I(VintageFlakes        ,0          ,1          ) /* internal flag */\
    DOIT_I(WhirlFactor          ,100        ,100        ) \
    DOIT_I(WindTimer            ,30         ,30         ) \
+   DOIT_I(IgnoreTop            ,0          ,0          ) \
+   DOIT_I(IgnoreBottom         ,0          ,0          ) \
    \
-   DOIT_S(SnowColor            ,"snow"           ,"snow"      ) \
-   DOIT_S(TreeColor            ,"chartreuse"     ,"chartreuse") \
+   DOIT_S(SnowColor            ,"#fffafa"  ,"#fffafa"  ) /* "snow" */       \
+   DOIT_S(TreeColor            ,"#7fff00"  ,"#7fff00"  ) /* "chartreuse" */ \
    DOIT_S(TreeType             ,"1,2,3,4,5,6,7," ,"0,"        ) \
    \
    DOIT_I(Anarchy              ,50  ,50 ) \
@@ -107,6 +120,7 @@
    DOIT_I(BirdsOnly            ,0   ,0  ) \
    DOIT_I(BirdsRestart         ,0   ,0  ) \
    DOIT_I(BirdsScale           ,100 ,100) \
+   DOIT_I(AttrSpace            ,40  ,40 ) \
    DOIT_I(BirdsSpeed           ,100 ,100) \
    DOIT_I(DisWeight            ,20  ,20 ) \
    DOIT_I(FollowWeight         ,30  ,30 ) \
@@ -118,5 +132,7 @@
    DOIT_I(ShowBirds            ,1   ,0  ) \
    DOIT_I(ViewingDistance      ,40  ,40 ) \
    \
-   DOIT_S(BirdsColor           ,"#361A07" ,"#361A07")
+   DOIT_S(BirdsColor           ,"#361A07" ,"#361A07") \
+   DOIT_S(BackgroundFile       ," "        ," ") \
+   DOIT_I(BlackBackground      ,0   ,0 )
 
