@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020,2021 Willem Vermin
+#-# 	      2019,2020,2021,2022 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -17,7 +17,10 @@
 #-# You should have received a copy of the GNU General Public License
 #-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #-# 
-*/
+ *
+ */
+#include <pthread.h>
+#include <semaphore.h>
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -27,6 +30,9 @@
 #include <math.h>
 #include <X11/Xlib.h>
 #include <X11/Intrinsic.h>
+#include <gsl/gsl_errno.h>
+#include <gsl/gsl_spline.h>
+#include <gsl/gsl_sort.h>
 #include "fallensnow.h"
 #include "utils.h"
 #include "windows.h"
@@ -36,24 +42,51 @@
 #include "blowoff.h"
 #include "wind.h"
 #include "debug.h"
+#include "safe_malloc.h"
+#include "spline_interpol.h"
 
 #define NOTACTIVE \
-   (Flags.BirdsOnly || !WorkspaceActive() || Flags.NoSnowFlakes)
-
+   (Flags.BirdsOnly || !WorkspaceActive() || Flags.NoSnowFlakes || (Flags.NoKeepSWin && Flags.NoKeepSBot))
 
 
 static void   drawquartcircle(int n, short int *y);  // nb: dimension of y > n+1
 static void   CreateSurfaceFromFallen(FallenSnow *f);
 static void   EraseFallenPixel(FallenSnow *fsnow,int x);
+static void   CreateDesh(FallenSnow *p);
+static int    do_change_deshes(void *dummy);
+static int    do_adjust_deshes(void *dummy);
+static void   swapsurfaces(void);
+static void  *do_fallen(void *d);
+static int    lock_swap(void);
+static int    unlock_swap(void);
+static void   check_fallen(void);
+
+// pop first element
+static int    PopFallenSnow(FallenSnow **list);
+
+static sem_t swap_sem;
+static sem_t fallen_sem;
+
+void fallen_sem_init()
+{
+   sem_init(&swap_sem,0,1);
+   sem_init(&fallen_sem,0,1);
+}
 
 void fallensnow_init()
 {
+   P("fallensnow_init\n");
    InitFallenSnow();
+   add_to_mainloop(PRIORITY_DEFAULT, time_change_bottom, do_change_deshes);
+   add_to_mainloop(PRIORITY_DEFAULT, time_adjust_bottom, do_adjust_deshes);
+   static pthread_t thread; 
+   pthread_create(&thread,NULL,do_fallen,NULL);
    P(" \n");
 }
 
 void UpdateFallenSnowAtBottom()
 {
+   // threads: locking by caller
    FallenSnow *fsnow = FindFallen(global.FsnowFirst, 0);
    if (fsnow)
       fsnow->y = global.SnowWinHeight;
@@ -63,6 +96,8 @@ void fallensnow_draw(cairo_t *cr)
 {
    if (NOTACTIVE)
       return;
+
+   lock_swap();
    FallenSnow *fsnow = global.FsnowFirst;
    while(fsnow)
    {
@@ -71,7 +106,7 @@ void fallensnow_draw(cairo_t *cr)
 	 P("fallensnow_draw %d %d\n",counter++,
 	       cairo_image_surface_get_width(fsnow->surface));
 	 P("fallensnow_draw: x:%d y:%d\n",fsnow->x,fsnow->y);
-	 cairo_set_source_surface (cr, fsnow->surface, fsnow->x, fsnow->y-fsnow->h+1);
+	 cairo_set_source_surface (cr, fsnow->surface, fsnow->x, fsnow->y-fsnow->h);
 	 my_cairo_paint_with_alpha(cr,ALPHA);
 	 fsnow->prevx = fsnow->x;
 	 fsnow->prevy = fsnow->y-fsnow->h+1;
@@ -80,58 +115,131 @@ void fallensnow_draw(cairo_t *cr)
       }
       fsnow = fsnow->next;
    }
+   unlock_swap();
 }
 
-void fallensnow_erase()
-{
-   if (NOTACTIVE)
-      return;
-   FallenSnow *fsnow = global.FsnowFirst;
-   while(fsnow)
-   {
-      if (HandleFallenSnow(fsnow)) 
-      {
-	 P("fallensnow_erase %d %d %d %d %d\n",counter++,
-	       fsnow->prevx, fsnow->prevy, fsnow->prevw, fsnow->prevh);
-
-	 myXClearArea(global.display,global.SnowWin,
-	       fsnow->prevx, fsnow->prevy, fsnow->prevw, fsnow->prevh, global.xxposures);
-      }
-      fsnow = fsnow->next;
-   }
-
-}
 
 void fallensnow_ui()
 {
    UIDO(MaxWinSnowDepth   , InitFallenSnow(); ClearScreen(); );
    UIDO(MaxScrSnowDepth   , 
-	 SetMaxScreenSnowDepth();
+	 SetMaxScreenSnowDepthWithLock();
 	 InitFallenSnow();
 	 ClearScreen();
        );
    UIDO(NoKeepSBot        , InitFallenSnow(); ClearScreen(); );
    UIDO(NoKeepSWin        , InitFallenSnow(); ClearScreen(); );
+   UIDO(IgnoreTop         ,                                  );
+   UIDO(IgnoreBottom      ,                                  );
 }
 
-int do_fallen(void *d)
+int lock_fallen() //Note: there is a macro Lock_fallen see fallensnow.h
+{
+   P("lock_fallen\n");
+   return sem_wait(&fallen_sem);
+}
+
+int unlock_fallen() //Note: there is a macro Unlock_fallen, see fallensnow.h
+{
+   P("unlock_fallen\n");
+   return sem_post(&fallen_sem);
+}
+
+// tries to get a lock on fallen_sem
+// if (*c)++ <= n, the function returns immediately,
+// the return value tells if the lock succeeded
+// 0: succes, else no success
+// if (*c)++ >n, sem_wait is used, the function returns 0 after getting
+// the lock. 
+// in both cases, *c is set to zero if the lock is obtained
+int lock_fallen_n(int n, int *c)  // see fallensnow.h for the macro Lock_fallen_n
+{
+   int rc;
+   if (*c < 0) 
+      *c = 0;
+   (*c)++;
+   if (*c > n)
+      rc = sem_wait(&fallen_sem);
+   else
+      rc = sem_trywait(&fallen_sem);
+   P("lock_fallen_n %d %d %d\n",n,*c,rc);
+   if(rc == 0)
+      *c = 0;
+   return rc;
+}
+
+void check_fallen()
+{
+   int i;
+   int rc = sem_getvalue(&fallen_sem,&i);
+   if(rc)
+   {
+      printf("error in get_semvalue()\n");
+      traceback();
+      exit(1);
+   }
+   if (i != 0)
+   {
+      printf("fallen_sem: %d\n",i);
+      traceback();
+      exit(1);
+   }
+}
+
+int lock_swap()
+{
+   return sem_wait(&swap_sem);
+}
+
+int unlock_swap()
+{
+   return sem_post(&swap_sem);
+}
+
+
+void *do_fallen(void *d)
 {
 
-   if (Flags.Done)
-      return FALSE;
-   if (NOTACTIVE)
-      return TRUE;
+   (void)d;
+   while(1)
+   {
+      if (Flags.Done)
+	 pthread_exit(NULL);
+      if (!NOTACTIVE)
+      {
+	 P("%d do_fallen\n",global.counter++);
+	 Lock_fallen();
 
+	 FallenSnow *fsnow = global.FsnowFirst;
+	 while(fsnow)
+	 {
+	    if (HandleFallenSnow(fsnow)) 
+	       DrawFallen(fsnow);
+	    fsnow = fsnow->next;
+	 }
+	 XFlush(global.display);
+
+	 swapsurfaces();
+
+	 Unlock_fallen();
+      }
+      usleep((useconds_t)(time_fallen*1000000));
+   }
+   return NULL;
+}
+
+void swapsurfaces()
+{
+   lock_swap();
    FallenSnow *fsnow = global.FsnowFirst;
    while(fsnow)
    {
-      if (HandleFallenSnow(fsnow)) 
-	 DrawFallen(fsnow);
-      fsnow = fsnow->next;
+      cairo_surface_t *s = fsnow->surface1;
+      fsnow->surface1    = fsnow->surface;
+      fsnow->surface     = s;
+      fsnow              = fsnow->next;
    }
-   XFlush(global.display);
-   return TRUE;
-   (void)d;
+   unlock_swap();
 }
 
 void drawquartcircle(int n, short int *y)  // nb: dimension of y > n+1
@@ -142,9 +250,64 @@ void drawquartcircle(int n, short int *y)  // nb: dimension of y > n+1
       y[i] = lrintf(sqrtf(n2 - i*i));
 }
 
+void CreateDesh(FallenSnow *p)
+{
+   // threads: locking by caller
+   int i;
+   int w           = p->w;
+   int h           = p->h;
+   int id          = p->win.id;
+   short int *desh = p->desh;
+#define N 6
+   double splinex[N];
+   double spliney[N];
+
+   randomuniqarray(splinex,N,0.0000001,NULL);
+   for (i=0; i<N; i++)
+   {
+      splinex[i] *= (w-1);
+      spliney[i] = drand48();
+   }
+
+   splinex[0] = 0;
+   splinex[N-1] = w-1;
+   if(id == 0) // bottom
+   {
+      spliney[0]   = 1.0;
+      spliney[N-1] = 1.0;
+   }
+   else
+   {
+      spliney[0]   = 0;
+      spliney[N-1] = 0;
+   }
+
+   double *x = (double *)malloc(w*sizeof(double));
+   double *y = (double *)malloc(w*sizeof(double));
+   for (i=0; i<w; i++)
+      x[i] = i;
+   spline_interpol(splinex, N, spliney, x, w, y);
+   for (i=0; i<w; i++)
+   {
+      desh[i] = h*y[i];
+      if (desh[i] < 2)
+	 desh[i] = 2;
+   }
+   free(x);
+   free(y);
+
+#if 0
+   FILE *ff = fopen("/tmp/desh","w");
+   for (i=0; i<w; i++)
+      fprintf(ff,"%d %d\n",i,desh[i]);
+   fclose(ff);
+#endif
+}
+
 // insert a node at the start of the list 
 void PushFallenSnow(FallenSnow **first, WinInfo *win, int x, int y, int w, int h) 
 {
+   // threads: locking by caller
    FallenSnow *p = (FallenSnow *)malloc(sizeof(FallenSnow));
    p->win        = *win;
    p->x          = x;
@@ -155,16 +318,20 @@ void PushFallenSnow(FallenSnow **first, WinInfo *win, int x, int y, int w, int h
    p->prevy      = 0;
    p->prevw      = 10;
    p->prevh      = 10;
-   p->w8         = ((w-1)/8+1)*8;
    p->acth       = (short int *)malloc(sizeof(*(p->acth))*w);
    p->desh       = (short int *)malloc(sizeof(*(p->desh))*w);
    p->surface    = cairo_image_surface_create(CAIRO_FORMAT_ARGB32,w,h);
+   //p->surface1   = cairo_image_surface_create(CAIRO_FORMAT_ARGB32,w,h);
+   p->surface1   = cairo_surface_create_similar (p->surface, CAIRO_CONTENT_COLOR_ALPHA, w, h);
 
-   cairo_t *cr = cairo_create(p->surface);
-   cairo_set_source_rgba(cr,1,0,0,0.0);
-   cairo_rectangle(cr,0,0,w,h);
-   cairo_fill(cr);
-   cairo_destroy(cr);
+   if(0)
+   {
+      cairo_t *cr = cairo_create(p->surface);
+      cairo_set_source_rgba(cr,1,0,0,0.0);
+      cairo_rectangle(cr,0,0,w,h);  // todo: surface is already clear?
+      cairo_fill(cr);
+      cairo_destroy(cr);
+   }
 
    int l = 0,i;
    for (i=0; i<w; i++)
@@ -176,21 +343,72 @@ void PushFallenSnow(FallenSnow **first, WinInfo *win, int x, int y, int w, int h
 	 l = 0;
    }
 
+   CreateDesh(p);
 
+   (void)drawquartcircle;
+#if 0
    if (w > h && win->id != 0)
    {
+      int i;
       drawquartcircle(h,&(p->desh[w-h-1]));
       for(i=0; i<=h; i++)
 	 p->desh[i] = p->desh[w-1-i];
    }
+#endif
 
    p->next  = *first;
    *first   = p;
 }
 
+
+// change to desired heights
+int do_change_deshes(void *dummy)
+{
+   (void)dummy;
+   static int lockcounter;
+   if(Lock_fallen_n(3,&lockcounter))
+      return TRUE;
+   FallenSnow *fsnow = global.FsnowFirst;
+   while(fsnow)
+   {
+      CreateDesh(fsnow);
+      fsnow = fsnow->next;
+   }
+   Unlock_fallen();
+   return TRUE;
+}
+
+int do_adjust_deshes(void *dummy)
+{
+   // threads: probably no need for lock, but to be sure:
+   Lock_fallen();
+   FallenSnow *fsnow = global.FsnowFirst;
+   while(fsnow)
+   {
+      int i;
+      int adjustments = 0;
+      for(i=0; i<fsnow->w; i++)
+      {
+	 int d = fsnow->acth[i] - fsnow->desh[i];
+	 if (d > 0)
+	 {
+	    int c = 1;
+	    adjustments++;
+	    fsnow->acth[i] -= c;
+	 }
+      }
+      P("adjustments: %d\n",adjustments);
+      fsnow = fsnow->next;
+   }
+   Unlock_fallen();
+   return TRUE;
+   (void)dummy;
+}
+
 // pop from list
 int PopFallenSnow(FallenSnow **list)
 {
+   // threads: locking by caller
    FallenSnow *next_node = NULL;
 
    if (*list == NULL) 
@@ -205,9 +423,12 @@ int PopFallenSnow(FallenSnow **list)
 // remove by id
 int RemoveFallenSnow(FallenSnow **list, Window id)
 {
+   // threads: locking by caller
    P("RemoveFallenSnow\n");
    if (*list == NULL)
+   {
       return 0;
+   }
 
    FallenSnow *fallen = *list;
    if (fallen->win.id == id)
@@ -223,7 +444,9 @@ int RemoveFallenSnow(FallenSnow **list, Window id)
    while (1)
    {
       if (fallen->next == NULL)
+      {
 	 return 0;
+      }
       scratch = fallen->next;
       if (scratch->win.id == id)
 	 break;
@@ -238,14 +461,17 @@ int RemoveFallenSnow(FallenSnow **list, Window id)
 
 void FreeFallenSnow(FallenSnow *fallen)
 {
+   // threads: locking by caller
    free(fallen->acth);
    free(fallen->desh);
    cairo_surface_destroy(fallen->surface);
+   cairo_surface_destroy(fallen->surface1);
    free(fallen);
 }
 
 FallenSnow *FindFallen(FallenSnow *first, Window id)
 {
+   // threads: locking by caller
    FallenSnow *fsnow = first;
    while(fsnow)
    {
@@ -255,6 +481,7 @@ FallenSnow *FindFallen(FallenSnow *first, Window id)
    }
    return NULL;
 }
+
 // print list
 void PrintFallenSnow(FallenSnow *list)
 {
@@ -273,17 +500,21 @@ void PrintFallenSnow(FallenSnow *list)
 
 void CleanFallenArea(FallenSnow *fsnow,int xstart,int w)
 {
+   // threads: locking by caller
    if(global.IsDouble)
       return;
    P("CleanFallenArea %d %d\n",counter++,global.IsDouble);
    int x = fsnow->prevx;
    int y = fsnow->prevy;
-   myXClearArea(global.display, global.SnowWin, x+xstart, y, w, fsnow->h+global.MaxSnowFlakeHeight, global.xxposures);
+   if (!global.IsDouble)
+      myXClearArea(global.display, global.SnowWin, x+xstart, y, 
+	    w, fsnow->h+global.MaxSnowFlakeHeight, global.xxposures);
 }
 
 // clean area for fallensnow with id
 void CleanFallen(Window id)
 {
+   // threads: locking by caller
    P("CleanFallen %#lx\n",id);
    FallenSnow *fsnow = global.FsnowFirst;
    // search the id
@@ -292,8 +523,6 @@ void CleanFallen(Window id)
       if(fsnow->win.id == id)
       {
 	 CleanFallenArea(fsnow,0,fsnow->w);
-	 //myXClearArea(global.display,global.SnowWin,
-	 //     fsnow->prevx, fsnow->prevy, fsnow->prevw, fsnow->prevh, global.xxposures);
 	 break;
       }
       fsnow = fsnow->next;
@@ -302,36 +531,173 @@ void CleanFallen(Window id)
 
 void CreateSurfaceFromFallen(FallenSnow *f)
 {
+   // threads: locking by caller
    P("createsurface %#10lx %d %d %d %d %d %d\n",f->id,f->x,f->y,f->w,f->h,
-	 cairo_image_surface_get_width(f->surface),
-	 cairo_image_surface_get_height(f->surface));
-   int i;
+	 cairo_image_surface_get_width(f->surface1),
+	 cairo_image_surface_get_height(f->surface1));
    GdkRGBA color;
 
-   // clear the surface (NB: CAIRO_OPERATOR_SOURCE)
-   cairo_t *cr = cairo_create(f->surface);
-   cairo_set_source_rgba(cr,0,0,0,0);
+   cairo_t *cr      = cairo_create(f->surface1);
+   int h            = f->h;
+   int w            = f->w;
+   short int *acth  = f->acth;
+   int id           = f->win.id;
+
+   cairo_set_antialias(cr, CAIRO_ANTIALIAS_DEFAULT);
    cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
-   cairo_rectangle(cr,0,0,f->w,f->h);
-   cairo_fill(cr);
-   cairo_set_line_width(cr,1);
-   cairo_set_antialias(cr,CAIRO_ANTIALIAS_NONE);
+
    gdk_rgba_parse(&color,Flags.SnowColor);
    cairo_set_source_rgb(cr,color.red, color.green, color.blue);
-   int h = f->h;
-   short int *acth = f->acth;
 
-   for (i=0; i<f->w; i++)
    {
-      cairo_move_to(cr,i,h-1);
-      cairo_line_to(cr,i,h-1-acth[i]);
+      // clear surface1
+      cairo_save(cr);
+      cairo_set_source_rgba(cr, 0, 0, 0, 0);
+      cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
+      cairo_paint (cr);
+      cairo_restore(cr);
    }
-   cairo_stroke(cr);
+
+   {
+      // compute averages for 10 points, draw spline through them
+      // and use that to draw fallensnow
+
+      const int m = 10;
+      int nav = 3+(w-2)/m;
+
+      double *av = (double *)malloc(nav*sizeof(double));
+      double *x  = (double *)malloc(nav*sizeof(double));
+
+      int i;
+      for (i=0; i<nav-3; i++)
+      {
+	 int j;
+	 double s = 0;
+	 for (j=0; j<m; j++)
+	    s += acth[m*i+j];
+	 av[i+1] = s/m;
+	 x[i+1]  = m*i + 0.5*m;
+      }
+      x[0]  = 0;
+      if(id == 0)
+	 av[0] = av[1];
+      else
+	 av[0] = 0;
+
+      int k    = nav - 3;
+      int mk   = m*k;
+      double s = 0;
+      for (i=mk; i<w; i++)
+	 s += acth[i];
+
+      av[k+1] = s/(w-mk);
+      x[k+1]  = mk + 0.5*(w-mk-1);
+
+      if(id == 0)
+	 av[nav-1] = av[nav-2];
+      else
+	 av[nav-1] = 0;
+      x[nav-1]  = w-1;
+
+      gsl_interp_accel *acc = gsl_interp_accel_alloc();
+      gsl_spline *spline    = gsl_spline_alloc(SPLINE_INTERP, nav);
+      gsl_spline_init(spline,x,av,nav);
+
+      cairo_set_line_width(cr,1);
+
+      cairo_set_antialias(cr,CAIRO_ANTIALIAS_DEFAULT);
+
+      if(0)
+      {
+	 cairo_move_to(cr,x[0],h-av[0]);
+	 for(i=0; i<w; i++)
+	    cairo_line_to(cr,i,h-gsl_spline_eval(spline, i, acc));
+	 cairo_line_to(cr,w-1,h);
+	 cairo_line_to(cr,0,h);
+	 cairo_close_path(cr);
+	 cairo_stroke_preserve(cr);
+	 cairo_fill(cr);
+      }
+      else
+      {
+	 // draw fallensnow: a move_to, followed by line_to's, followed by close_path and fill.
+	 // to prevent a permanent bottom snow-line, even if there has no snow fallen on
+	 // certain parts, only handle regions where snow has been fallen.
+	 enum{searching, drawing};
+	 int state = searching; // searching for acth[] > 0
+	 int i;
+	 int startpos;
+	 for (i=0; i<w; ++i)
+	 {
+	    int val = gsl_spline_eval(spline,i,acc);
+	    switch(state)
+	    {
+	       case searching:
+		  if (val != 0)
+		  {
+		     startpos = i;
+		     cairo_move_to(cr, i, h);
+		     cairo_line_to(cr, i, h);
+		     cairo_line_to(cr, i, h - val);
+		     state = drawing;
+		  }
+		  break;
+	       case drawing:
+		  cairo_line_to(cr, i, h - val);
+		  if (val == 0 || i == w-1)
+		  {
+		     cairo_line_to(cr, i, h);
+		     cairo_line_to(cr, startpos, h);
+		     cairo_close_path(cr);
+		     cairo_stroke_preserve(cr);
+		     cairo_fill(cr);
+		     state = searching;
+		  }
+		  break;
+	    }
+	 }
+      }
+      if(0)
+      {
+	 // draw averages
+	 int i;
+	 cairo_save(cr);
+	 cairo_set_source_rgba(cr,1,0,0,1);
+	 for (i=0; i<nav; i++)
+	    cairo_rectangle(cr,x[i],h-av[i]-4,4,4);
+	 cairo_fill(cr);
+	 cairo_restore(cr);
+      }
+      gsl_spline_free (spline);
+      gsl_interp_accel_free (acc);
+
+      free(x);
+      free(av);
+   }
+
+   if(0)
+   {
+      // draw max height of fallensnow (using f->desh)
+      cairo_save(cr);
+      int j;
+
+      cairo_set_operator(cr,CAIRO_OPERATOR_OVER);
+      cairo_set_source_rgba(cr,1,0,0,1);
+      for(j=0; j<w; j++)
+      {
+	 cairo_rectangle(cr,j,h-f->desh[j],1,1);
+      }
+      cairo_fill(cr);
+      cairo_fill(cr);
+      cairo_restore(cr);
+   }
+
    cairo_destroy(cr);
 }
 
 void DrawFallen(FallenSnow *fsnow)
 {
+   // threads: locking done by caller
    if(fsnow->win.id == 0 || (!fsnow->win.hidden &&
 	    (fsnow->win.ws == global.CWorkSpace || fsnow->win.sticky)))
    {
@@ -346,7 +712,8 @@ void DrawFallen(FallenSnow *fsnow)
 	    int xfront = global.SantaX+global.SantaWidth - fsnow->x;
 	    // determine back of Santa in fsnow, Santa can move backwards in strong wind
 	    int xback = xfront - global.SantaWidth;
-	    const int clearing = 1;
+	    // clearing determines the amount of generated ploughing snow
+	    const int clearing = 10;
 	    float vy = -1.5*global.ActualSantaSpeed; 
 	    if(vy > 0) vy = -vy;
 	    if (vy < -100.0)
@@ -362,22 +729,29 @@ void DrawFallen(FallenSnow *fsnow)
 	 }
       }
       CreateSurfaceFromFallen(fsnow);
-      // drawing is handled in fallensnow-draw
+      // drawing is handled in fallensnow_draw
    }
 }
 
 void GenerateFlakesFromFallen(FallenSnow *fsnow, int x, int w, float vy)
 {
+   // threads: locking by caller
    P("GenerateFlakes %d %d %d %f\n",counter++,x,w,vy);
    if (!Flags.BlowSnow || Flags.NoSnowFlakes)
       return;
    // animation of fallen fallen snow
    // x-values x..x+w are transformed in flakes, vertical speed vy
    int i;
-   int ifirst = x; if (ifirst < 0) ifirst = 0;
-   if (ifirst > fsnow->w) ifirst = fsnow->w;
-   int ilast  = x+w; if(ilast < 0) ilast = 0;
-   if (ilast > fsnow->w) ilast = fsnow->w;
+   int ifirst = x; 
+   if (ifirst < 0) 
+      ifirst = 0;
+   if (ifirst > fsnow->w) 
+      ifirst = fsnow->w;
+   int ilast  = x+w; 
+   if(ilast < 0) 
+      ilast = 0;
+   if (ilast > fsnow->w) 
+      ilast = fsnow->w;
    P("ifirst ilast: %d %d %d %d\n",ifirst,ilast,w,w<global.MaxSnowFlakeWidth?w:global.MaxSnowFlakeWidth);
    P("maxheight: %d maxw: %d\n",global.MaxSnowFlakeHeight,global.MaxSnowFlakeWidth);
    //for (i=ifirst; i<ilast; i+=w<global.MaxSnowFlakeHeight?w:global.MaxSnowFlakeWidth)
@@ -415,13 +789,15 @@ void GenerateFlakesFromFallen(FallenSnow *fsnow, int x, int w, float vy)
 
 void EraseFallenPixel(FallenSnow *fsnow, int x)
 {
+   // threads: locking by caller
    if(fsnow->acth[x] > 0)
    {
       if(!global.IsDouble)
       {
 	 int x1 = fsnow->x + x;
 	 int y1 = fsnow->y - fsnow->acth[x];
-	 myXClearArea(global.display, global.SnowWin, x1 , y1, 1, 1, global.xxposures);     
+	 if(!global.IsDouble)
+	    myXClearArea(global.display, global.SnowWin, x1 , y1, 1, 1, global.xxposures);     
       }
       fsnow->acth[x]--;
    }
@@ -429,6 +805,7 @@ void EraseFallenPixel(FallenSnow *fsnow, int x)
 
 void InitFallenSnow()
 {
+   Lock_fallen();
    while (global.FsnowFirst)
       PopFallenSnow(&global.FsnowFirst);
    // create fallensnow on bottom of screen:
@@ -437,12 +814,16 @@ void InitFallenSnow()
 
    PushFallenSnow(&global.FsnowFirst, NullWindow, 0, global.SnowWinHeight, global.SnowWinWidth, global.MaxScrSnowDepth);
    free(NullWindow);
+
+   Unlock_fallen();
+   (void)check_fallen;  // to prevent warning about unused check_fallen
 }
 
 // removes some fallen snow from fsnow, w pixels. If fallensnowheight < h: no removal
 // also add snowflakes
 void UpdateFallenSnowWithWind(FallenSnow *fsnow, int w, int h)
 {
+   // threads: locking by caller
    int i;
    int x = randint(fsnow->w - w);
    for(i=x; i<x+w; i++)
@@ -454,11 +835,11 @@ void UpdateFallenSnowWithWind(FallenSnow *fsnow, int w, int h)
 	    int j, jmax = BlowOff();
 	    for (j=0; j< jmax; j++)
 	    {
-	       Snow *flake       = MakeFlake(0);
+	       Snow *flake       = MakeFlake(-1);
 	       flake->rx         = fsnow->x + i;
-	       flake->ry         = fsnow->y - fsnow->acth[i] - drand48()*8;
-	       flake->vx         = fsignf(global.NewWind)*global.WindMax;
-	       flake->vy         = -5;
+	       flake->ry         = fsnow->y - fsnow->acth[i] - drand48()*4;
+	       flake->vx         = 0.25*fsignf(global.NewWind)*global.WindMax;
+	       flake->vy         = -10;
 	       flake->cyclic     = (fsnow->win.id == 0); // not cyclic for Windows, cyclic for bottom
 	       P("%d:\n",counter++);
 	    }
@@ -467,8 +848,16 @@ void UpdateFallenSnowWithWind(FallenSnow *fsnow, int w, int h)
       }
 }
 
+void SetMaxScreenSnowDepthWithLock()
+{
+   Lock_fallen();
+   SetMaxScreenSnowDepth();
+   Unlock_fallen();
+}
+
 void SetMaxScreenSnowDepth()
 {
+   // threads: locking by caller
    global.MaxScrSnowDepth = Flags.MaxScrSnowDepth;
    if (global.MaxScrSnowDepth> (global.SnowWinHeight-SNOWFREE)) {
       printf("** Maximum snow depth set to %d\n", global.SnowWinHeight-SNOWFREE);
