@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020,2021 Willem Vermin
+#-# 	      2019,2020,2021,2022 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -17,7 +17,8 @@
 #-# You should have received a copy of the GNU General Public License
 #-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #-# 
-*/
+ *
+ */
 /*
    And in a vocoded voice it sounds:
    Xsnow zwei-tausend
@@ -41,6 +42,7 @@
  * Reals dealing with time are declared as double. 
  * Other reals as float
  */
+#include <pthread.h>
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -55,6 +57,7 @@
 #include <stdio.h>
 #include <string.h>
 
+#include "aurora.h"
 #include "birds.h"
 #include "clocks.h"
 #include "docs.h"
@@ -62,7 +65,7 @@
 #include "fallensnow.h"
 #include "flags.h"
 #include "mainstub.h"
-#include "meteo.h"
+#include "meteor.h"
 #include "moon.h"
 #include "Santa.h"
 #include "scenery.h"
@@ -80,6 +83,8 @@
 #include "debug.h"
 #include "treesnow.h"
 #include "loadmeasure.h"
+#include "selfrep.h"
+#include "safe_malloc.h"
 
 #include "vroot.h"
 
@@ -137,6 +142,7 @@ static int    HandleX11Cairo(void);
 static int    StartWindow(void);
 static void   SetWindowScale(void);
 static void   movewindow(void);
+static void   GetDesktopSession(void);
 
 
 // callbacks
@@ -165,8 +171,9 @@ static gboolean     on_draw_event(GtkWidget *widget, cairo_t *cr, gpointer user_
  *   - scenery (trees etc)
  *   - stars
  *   - moon + halo
- *   - meteorites
+ *   - meteors
  *   - birds
+ *   - aurora
  *
  * Selection of window to draw in
  * ==============================
@@ -244,11 +251,17 @@ int main_c(int argc, char *argv[])
    signal(SIGINT,  SigHandler);
    signal(SIGTERM, SigHandler);
    signal(SIGHUP,  SigHandler);
-   srand48((long int)(wallcl()*1.0e6));
-
+   P("srand %d\n",(int)(fmod(wallcl()*1.0e6,1.0e8)));
+   srand48((int)(fmod(wallcl()*1.0e6,1.0e8)));
 
    memset((void *)&global,0,sizeof(global));
 
+   XInitThreads();
+
+   fallen_sem_init();
+   aurora_sem_init();
+   birds_sem_init();
+
    global.counter            = 0;
    global.cpufactor          = 1.0;
    global.WindowScale        = 1.0;
@@ -275,8 +288,9 @@ int main_c(int argc, char *argv[])
 
    global.Wind               = 0;
    global.Direction          = 0;
-   global.WindMax            = 100.0;
-   global.NewWind = 0;
+   //global.WindMax            = 100.0;
+   global.WindMax            = 500.0;
+   global.NewWind            = 100.0;
 
    global.HaltedByInterrupt  = 0;
    global.Message[0]         = 0;
@@ -315,6 +329,24 @@ int main_c(int argc, char *argv[])
 	 PrintVersion();
 	 return 0;
       }
+      else if (!strcmp(arg, "-changelog"))
+      {
+	 docs_changelog();
+	 return 0;
+      }
+#ifdef SELFREP
+      else if (!strcmp(arg, "-selfrep"))
+      {
+	 selfrep();
+	 return 0;
+      }
+#endif
+   }
+   GetDesktopSession();
+   if(!strncasecmp(global.DesktopSession,"bspwm",5))
+   {
+      printf("For optimal resuts, add to your bspwmrc:\n");
+      printf("   bspc rule -a Xsnow state=floating border=off\n");
    }
    printf("Xsnow running in GTK version: %s\n",ui_gtk_version());
 
@@ -350,6 +382,9 @@ int main_c(int argc, char *argv[])
    if (!Flags.NoConfig)
       WriteFlags();
 
+   if (Flags.BelowAllForce)
+      Flags.BelowAll = 0;
+
    if(Flags.CheckGtk && !ui_checkgtk() && !Flags.NoMenu)
    {
       printf("Xsnow needs gtk version >= %s, found version %s\n",ui_gtk_required(),ui_gtk_version());
@@ -374,6 +409,17 @@ int main_c(int argc, char *argv[])
    global.White = WhitePixel(global.display, screen);
 
 
+   { // if somebody messed up colors in .xsnowrc:
+      if (!ValidColor(Flags.TreeColor))
+	 Flags.TreeColor = strdup(DefaultFlags.TreeColor); 
+      if (!ValidColor(Flags.SnowColor))
+	 Flags.SnowColor = strdup(DefaultFlags.SnowColor); 
+      if (!ValidColor(Flags.BirdsColor))
+	 Flags.BirdsColor = strdup(DefaultFlags.BirdsColor); 
+      WriteFlags();
+   }
+
+
    InitSnowOnTrees();
 
    if (global.display == NULL) {
@@ -423,32 +469,6 @@ int main_c(int argc, char *argv[])
       return 1;
    }
 
-   Flags.Done = 0;
-   windows_init();
-   moon_init();
-   Santa_init();
-   birds_init();
-   scenery_init();
-   snow_init();
-   meteo_init();
-   wind_init();
-   stars_init();
-   fallensnow_init();
-   blowoff_init();
-   treesnow_init();
-   loadmeasure_init();
-
-   add_to_mainloop(PRIORITY_DEFAULT, time_displaychanged, do_displaychanged     );
-   add_to_mainloop(PRIORITY_DEFAULT, time_event,          do_event              );
-   add_to_mainloop(PRIORITY_DEFAULT, time_testing,        do_testing            );
-   add_to_mainloop(PRIORITY_DEFAULT, 0.5,                 do_display_dimensions );
-   add_to_mainloop(PRIORITY_HIGH,    time_ui_check,       do_ui_check           );
-   add_to_mainloop(PRIORITY_DEFAULT, time_show_range_etc, do_show_range_etc     );
-
-   if (Flags.StopAfter > 0)
-      add_to_mainloop(PRIORITY_DEFAULT, Flags.StopAfter, do_stopafter);
-
-   HandleCpuFactor();
 
 #define DOIT_I(x,d,v) OldFlags.x = Flags.x;
 #define DOIT_L(x,d,v) DOIT_I(x,d,v);
@@ -461,7 +481,6 @@ int main_c(int argc, char *argv[])
 
    BlackPix = AllocNamedColor(BlackColor, global.Black);
 
-   // events
    if(global.Desktop)
    {
       XSelectInput(global.display, global.Rootwindow, StructureNotifyMask| SubstructureNotifyMask);
@@ -471,18 +490,46 @@ int main_c(int argc, char *argv[])
 
    ClearScreen();   // without this, no snow, scenery etc. in KDE; this seems to be a vintage comment
 
-   if(!Flags.NoMenu)
+   if(!Flags.NoMenu && !global.XscreensaverMode)
    {
       ui();
       ui_gray_erase(global.Trans);
       ui_set_sticky(Flags.AllWorkspaces);
-      add_to_mainloop(PRIORITY_DEFAULT, 2.0, do_show_desktop_type);
+      add_to_mainloop(PRIORITY_DEFAULT, time_desktop_type, do_show_desktop_type);
    }
+   Flags.Done = 0;
+   windows_init();
+   moon_init();
+   aurora_init();
+   Santa_init();
+   birds_init();
+   scenery_init();
+   snow_init();
+   meteor_init();
+   wind_init();
+   stars_init();
+   blowoff_init();
+   treesnow_init();
+   loadmeasure_init();
+   fallensnow_init();
+
+   add_to_mainloop(PRIORITY_DEFAULT, time_displaychanged,     do_displaychanged     );
+   add_to_mainloop(PRIORITY_DEFAULT, time_event,              do_event              );
+   add_to_mainloop(PRIORITY_DEFAULT, time_testing,            do_testing            );
+   add_to_mainloop(PRIORITY_DEFAULT, time_display_dimensions, do_display_dimensions );
+   add_to_mainloop(PRIORITY_HIGH,    time_ui_check,           do_ui_check           );
+   add_to_mainloop(PRIORITY_DEFAULT, time_show_range_etc,     do_show_range_etc     );
+
+   if (Flags.StopAfter > 0)
+      add_to_mainloop(PRIORITY_DEFAULT, Flags.StopAfter, do_stopafter);
+
+   HandleCpuFactor();
+
+   fflush(stdout);
 
    // main loop
    gtk_main();
 
-
    if (SnowWinName) free(SnowWinName);
 
    XClearWindow(global.display, global.SnowWin);
@@ -505,7 +552,7 @@ int main_c(int argc, char *argv[])
 
 void set_below_above()
 {
-   P("%d set_below_above\n",global.counter++);
+   P("%d set_below_above %d\n",global.counter++,Flags.BelowAll);
    XWindowChanges changes;
    // to be sure: we do it in gtk mode and window mode
    if (Flags.BelowAll)
@@ -545,6 +592,38 @@ void X11WindowById(Window *xwin, char **xwinname)
    return;
 }
 
+void GetDesktopSession()
+{
+   if (global.DesktopSession == NULL)
+   {
+      char *desktopsession = NULL;
+      const char *desktops[] = {
+	 "DESKTOP_SESSION",
+	 "XDG_SESSION_DESKTOP",
+	 "XDG_CURRENT_DESKTOP",
+	 "GDMSESSION",
+	 NULL
+      };
+
+      int i;
+      for (i=0; desktops[i]; i++)
+      {
+	 desktopsession = getenv(desktops[i]);
+	 if (desktopsession)
+	    break;
+      }
+      if (desktopsession)
+	 printf("Detected desktop session: %s\n",desktopsession);
+      else
+      {
+	 printf("Could not determine desktop session\n");
+	 desktopsession = (char *)"unknown_desktop_session";
+      }
+
+      global.DesktopSession = strdup(desktopsession);
+   }
+}
+
 int StartWindow()
 {
    int X11cairo     = 0;
@@ -554,6 +633,7 @@ int StartWindow()
    global.Desktop   = 0;
    global.UseDouble = 0;
    global.IsDouble  = 0;
+   global.XscreensaverMode = 0;
 
    global.Rootwindow = DefaultRootWindow(global.display);
    Window xwin;
@@ -572,6 +652,7 @@ int StartWindow()
       global.SnowWin = global.Rootwindow;
       if (getenv("XSCREENSAVER_WINDOW"))
       {
+	 global.XscreensaverMode  = 1;
 	 global.SnowWin    = strtol(getenv("XSCREENSAVER_WINDOW"),NULL,0);
 	 global.Rootwindow = global.SnowWin;
       }
@@ -582,6 +663,12 @@ int StartWindow()
       // default behaviour
       // try to create a transparent clickthrough window
       GtkWidget *gtkwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+      gtk_widget_set_can_focus(gtkwin, TRUE);
+      gtk_window_set_decorated(GTK_WINDOW(gtkwin), FALSE);
+      gtk_window_set_type_hint(GTK_WINDOW(gtkwin), GDK_WINDOW_TYPE_HINT_POPUP_MENU);
+
+
+      GdkWindow *gdkwin; (void)gdkwin;
       int rc = make_trans_window(gtkwin,
 	    1,                   // full screen 
 	    Flags.AllWorkspaces, // sticky 
@@ -599,9 +686,16 @@ int StartWindow()
 	 GtkWidget *drawing_area = gtk_drawing_area_new();
 	 gtk_container_add(GTK_CONTAINER(TransA), drawing_area);
 	 g_signal_connect(TransA, "draw", G_CALLBACK(on_draw_event), NULL);
+	 P("calling set_below_above\n");
 	 set_below_above();
 	 global.SnowWin = xwin;
 	 printf("Using transparent window\n");
+	 if(!strncasecmp(global.DesktopSession,"fvwm",4) ||
+	       !strncasecmp(global.DesktopSession,"lxqt",4))
+	 {
+	    printf("The transparent snow-window is probably not click-through, alas..\n");
+	 }
+
       }
       else
       {
@@ -609,42 +703,12 @@ int StartWindow()
 	 X11cairo = 1;
 	 // use rootwindow, pcmanfm or Desktop:
 	 printf("Cannot create transparent window\n");
-	 if (global.DesktopSession == NULL)
-	 {
-	    char *desktopsession = NULL;
-	    const char *desktops[] = {
-	       "DESKTOP_SESSION",
-	       "XDG_SESSION_DESKTOP",
-	       "XDG_CURRENT_DESKTOP",
-	       "GDMSESSION",
-	       NULL
-	    };
-
-	    int i;
-	    for (i=0; desktops[i]; i++)
-	    {
-	       desktopsession = getenv(desktops[i]);
-	       if (desktopsession)
-		  break;
-	    }
-	    if (desktopsession)
-	       printf("Detected desktop session: %s\n",desktopsession);
-	    else
-	    {
-	       printf("Could not determine desktop session\n");
-	       desktopsession = (char *)"unknown_desktop_session";
-	    }
-
-	    global.DesktopSession = strdup(desktopsession);
-
-	    if (!strcasecmp(global.DesktopSession,"enlightenment"))
-	       printf("NOTE: xsnow will probably run, but some glitches are to be expected.\n");
-	    else if(!strcasecmp(global.DesktopSession,"twm"))
-	       printf("NOTE: you probably need to tweak 'Lift snow on windows' in the 'settings' panel.\n");
-	 }
+	 if (!strcasecmp(global.DesktopSession,"enlightenment"))
+	    printf("NOTE: xsnow will probably run, but some glitches are to be expected.\n");
+	 if(!strcasecmp(global.DesktopSession,"twm"))
+	    printf("NOTE: you probably need to tweak 'Lift snow on windows' in the 'settings' panel.\n");
 	 // if envvar DESKTOP_SESSION == LXDE, search for window with name pcmanfm
-	 if (global.DesktopSession != NULL && 
-	       !strncmp(global.DesktopSession,"LXDE",4) && 
+	 if (!strncmp(global.DesktopSession,"LXDE",4) && 
 	       (xwin = FindWindowWithName(global.display,"pcmanfm")))
 	 {
 	    printf("LXDE session found, using window 'pcmanfm'.\n");
@@ -678,7 +742,8 @@ int StartWindow()
       SnowWinName = strdup("no name");
    XFree(x.value);
    InitDisplayDimensions();
-   I("Snowing in %#lx: %s %d+%d %dx%d\n",global.SnowWin,SnowWinName,global.SnowWinX,global.SnowWinY,global.SnowWinWidth,global.SnowWinHeight);
+   printf("Snowing in %#lx: %s %d+%d %dx%d\n",global.SnowWin,SnowWinName,global.SnowWinX,global.SnowWinY,global.SnowWinWidth,global.SnowWinHeight);
+   fflush(stdout);
 
    Xorig = global.SnowWinX;
    Yorig = global.SnowWinY;
@@ -686,6 +751,8 @@ int StartWindow()
    movewindow();
 
    SetWindowScale();
+   if (global.XscreensaverMode && !Flags.BlackBackground)
+      SetBackground();
 
    return TRUE;
 }
@@ -716,7 +783,7 @@ int HandleX11Cairo()
       CairoSurface = cairo_xlib_surface_create(global.display, backBuf, visual, w, h);
       global.UseDouble = 1;
       global.IsDouble  = 1;
-      printf("Using double buffer: %#lx.\n",backBuf);
+      printf("Using double buffer: %#lx. %dx%d\n",backBuf,w,h);
       rcv = TRUE;
    }
 #endif
@@ -728,6 +795,7 @@ int HandleX11Cairo()
 	 printf(" because double buffering is not available on this system\n");
       else
 	 printf(" on your request.\n");
+      printf("NOTE: expect some flicker.\n");
       rcv = FALSE;
    }
 
@@ -777,28 +845,31 @@ int do_ui_check(void *d)
    scenery_ui();
    birds_ui();
    snow_ui();
-   meteo_ui();
+   meteor_ui();
    wind_ui();
    stars_ui();
    fallensnow_ui();
    blowoff_ui();
    treesnow_ui();
    moon_ui();
+   aurora_ui();
    ui_ui();
 
-   UIDO (CpuLoad             , HandleCpuFactor();               );
-   UIDO (Transparency        ,                                  );
-   UIDO (Scale               ,                                  );
-   UIDO (OffsetS             , DisplayDimensions();             );
-   UIDO (OffsetY             , UpdateFallenSnowRegions();       );
-   UIDO (NoFluffy            , ClearScreen();                   );
-   UIDO (AllWorkspaces       , DoAllWorkspaces();               );
-   UIDO (BelowAll            , set_below_above();               );
+   UIDO (CpuLoad             , HandleCpuFactor();                 );
+   UIDO (Transparency        ,                                    );
+   UIDO (Scale               ,                                    );
+   UIDO (OffsetS             , DisplayDimensions();               );
+   UIDO (OffsetY             , UpdateFallenSnowRegionsWithLock(); );
+   UIDO (NoFluffy            , ClearScreen();                     );
+   UIDO (AllWorkspaces       , DoAllWorkspaces();                 );
+   UIDO (BelowAll            , set_below_above();                 );
+   UIDOS(BackgroundFile      ,                                    );
+   UIDO (BlackBackground     ,                                    );
 
    if (Flags.Changes > 0)
    {
-      P("WriteFlags\n");
       WriteFlags();
+      set_buttons();
       P("-----------Changes: %d\n",Flags.Changes);
    }
    Flags.Changes = 0;
@@ -887,12 +958,13 @@ int do_event(void *d)
 	 } 
       }
    }  
+
    return TRUE;
 }
 
 void RestartDisplay()
 {
-   P("Restartdisplay: %d W: %d H: %d\n",counter++,global.SnowWinWidth,global.SnowWinHeight);
+   P("Restartdisplay: %d W: %d H: %d\n",global.counter++,global.SnowWinWidth,global.SnowWinHeight);
    InitFallenSnow();
    init_stars();
    EraseTrees();
@@ -954,7 +1026,7 @@ int do_testing(void *d)
    XWindowAttributes attr;
    XGetWindowAttributes(global.display,global.SnowWin,&attr);
 
-   P("%d wxh %d %d %d %d    %d %d %d %d \n",counter,SnowWinX,SnowWinY,global.SnowWinWidth,global.SnowWinHeight,attr.x,attr.y,attr.width,attr.height);
+   P("%d wxh %d %d %d %d    %d %d %d %d \n",global.counter,global.SnowWinX,global.SnowWinY,global.SnowWinWidth,global.SnowWinHeight,attr.x,attr.y,attr.width,attr.height);
 
    return TRUE;
 }
@@ -997,14 +1069,27 @@ gboolean on_draw_event(GtkWidget *widget, cairo_t *cr, gpointer user_data)
 
 int do_drawit(void *cr)
 {
-   P("do_drawit %d %p\n",counter++,cr);
+   P("do_drawit %d %p\n",global.counter++,cr);
    drawit((cairo_t*)cr);
    return TRUE;
 }
 
+
 void drawit(cairo_t *cr)
 {
-   P("drawit %d %p\n",counter++,(void *)cr);
+   static int counter = 0;
+   P("drawit %d %p\n",global.counter++,(void *)cr);
+   // Due to instabilities at the start of xsnow: spurious detection of
+   //   user intervention; resizing of screen; etc.
+   //   placement of scenery, stars etc is repeated a few times at the
+   //   start of xsnow.
+   // This is not harmful, but a bit annoying, so we do not draw anything 
+   //   the first few times this function is called.
+   if (counter*time_draw_all < 1.5)
+   {
+      counter++;
+      return;
+   }
 
    if (Flags.Done)
       return;
@@ -1024,18 +1109,30 @@ void drawit(cairo_t *cr)
       stars_erase(); // not really necessary
       birds_erase(0);
       snow_erase(1);
+      aurora_erase();
       XFlush(global.display);
    }
 
+
+   /* clear snowwindow test */
+   /*
+      cairo_set_source_rgba(cr, 0, 0, 0, 1);
+      cairo_set_line_width(cr, 1);
+
+      cairo_rectangle(cr, 0, 0, global.SnowWinWidth, global.SnowWinHeight);
+      cairo_fill(cr);
+      */
+
    int skipit = Flags.BirdsOnly || !WorkspaceActive();
 
    if (!skipit)
    {
       stars_draw(cr);
-      meteo_draw(cr);
 
       P("moon\n");
       moon_draw(cr);
+      aurora_draw(cr);
+      meteor_draw(cr);
       scenery_draw(cr);
    }
 
@@ -1055,7 +1152,6 @@ void drawit(cairo_t *cr)
    snow_draw(cr);
 
    XFlush(global.display);
-
 }
 
 void SetWindowScale()
@@ -1080,28 +1176,31 @@ int do_display_dimensions(void *d)
       return TRUE;
    SnowWinChanged = 0;
    static int prevw = 0, prevh = 0;
-   P("%d do_display_dimensions %d %d\n",counter++,global.SnowWinWidth,global.SnowWinHeight);
+   P("%d do_display_dimensions %dx%d %dx%d\n",global.counter++,global.SnowWinWidth,global.SnowWinHeight,prevw,prevh);
    DisplayDimensions();
    if (prevw != global.SnowWinWidth || prevh != global.SnowWinHeight)
    {
       // if(global.X11cairo) // let op
       if (!global.Trans)
       {
+	 P("global.Trans %d\n",global.Trans);
 	 HandleX11Cairo();
-	 RestartDisplay();
+	 //RestartDisplay();
       }
+      RestartDisplay();
       prevw = global.SnowWinWidth;
       prevh = global.SnowWinHeight;
       SetWindowScale();
    }
    return TRUE;
+   fflush(stdout);
 }
 
 int do_draw_all(gpointer widget)
 {
    if (Flags.Done)
       return FALSE;
-   P("do_draw_all %d %p\n",counter++,(void *)widget);
+   P("do_draw_all %d %p\n",global.counter++,(void *)widget);
 
    // this will result in a call off on_draw_event():
    gtk_widget_queue_draw(GTK_WIDGET(widget));
@@ -1112,20 +1211,14 @@ int do_draw_all(gpointer widget)
 // handle callbacks for things whose timings depend on cpufactor
 void HandleCpuFactor()
 {
-   static guint fallen_id=0;
-
    // re-add things whose timing is dependent on cpufactor
    if (Flags.CpuLoad <= 0)
       global.cpufactor = 1;
    else
       global.cpufactor = 100.0/Flags.CpuLoad;
 
-   if (fallen_id)
-      g_source_remove(fallen_id);
-
-   fallen_id = add_to_mainloop(PRIORITY_DEFAULT, time_fallen, do_fallen);
    P("handlecpufactor %f %f %d\n",oldcpufactor,cpufactor,counter++);
-   add_to_mainloop(PRIORITY_HIGH, 0.2 , do_initsnow);  // remove flakes
+   add_to_mainloop(PRIORITY_HIGH, time_init_snow , do_initsnow);  // remove flakes
 
    restart_do_draw_all();
 }
