@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020,2021 Willem Vermin
+#-# 	      2019,2020,2021,2022 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #-# 
 */
 
+#include <pthread.h>
 #include <stdio.h>
 #include <gtk/gtk.h>
 #include <stdlib.h>
@@ -40,6 +41,7 @@
 #include "ui.h"
 #include "blowoff.h"
 #include "treesnow.h"
+#include "safe_malloc.h"
 
 #define NOTACTIVE \
    (Flags.BirdsOnly || !WorkspaceActive() || Flags.NoSnowFlakes)
@@ -108,7 +110,7 @@ void snow_init()
    InitSnowSpeedFactor();
    add_to_mainloop(PRIORITY_DEFAULT, time_genflakes,      do_genflakes       );
    add_to_mainloop(PRIORITY_DEFAULT, time_flakecount,     do_show_flakecount );
-   add_to_mainloop(PRIORITY_DEFAULT, 0.2,                 do_SwitchFlakes    );
+   add_to_mainloop(PRIORITY_DEFAULT, time_switchflakes,   do_SwitchFlakes    );
 
    // now we would like to be able to get rid of the snow xpms:
    /*
@@ -136,6 +138,7 @@ void snow_ui()
    UIDO (SnowFlakesFactor               , InitFlakesPerSecond();   );
    UIDOS(SnowColor                      ,
 	 InitSnowColor();
+	 //InitFallenSnow(); 
 	 ClearScreen();                                            );
    UIDO (SnowSpeedFactor                , InitSnowSpeedFactor();   );
    UIDO (FlakeCountMax                  ,                          );
@@ -178,6 +181,7 @@ void init_snow_pix()
       g_clear_object(&pixbuf);
       g_clear_object(&pixbufscaled);
    }
+   global.fluffpix = &snowPix[MaxFlakeTypes-1];
 }
 
 int snow_draw(cairo_t *cr)
@@ -191,7 +195,6 @@ int snow_draw(cairo_t *cr)
    while( (flake = (Snow *)set_next()) )
    {
       P("snow_draw %d %f\n",counter++,ALPHA);
-      //cairo_set_source_surface (cr, snow_surfaces[flake->whatFlake], flake->rx, flake->ry);
       cairo_set_source_surface (cr, snowPix[flake->whatFlake].surface, flake->rx, flake->ry);
       double alpha = ALPHA;
       if (flake->fluff)
@@ -319,11 +322,18 @@ int do_UpdateSnowFlake(Snow *flake)
    //
    if (!Flags.NoWind)
    {
-      flake->vx += FlakesDT*flake->wsens*(global.NewWind - flake->vx)/flake->m;
+      P("flakevx1: %p %f\n",(void*)flake,flake->vx);
+      float f = FlakesDT*flake->wsens/flake->m;
+      if (f > 0.9)  f =  0.9;
+      if (f < -0.9) f = -0.9;
+      flake->vx += f*(global.NewWind - flake->vx);
       static float speedxmaxes[] = {100.0, 300.0, 600.0,};
-      float speedxmax = speedxmaxes[global.Wind];
-      if(flake->vx > speedxmax) flake->vx = speedxmax;
+      float speedxmax = 2*speedxmaxes[global.Wind];
+      if(fabs(flake->vx)>speedxmax)
+	 P("flakevx:  %p %f %f %f %f %f %f\n",(void*)flake,flake->vx,FlakesDT,flake->wsens,global.NewWind,flake->vx,flake->m);
+      if(flake->vx > speedxmax)  flake->vx =  speedxmax;
       if(flake->vx < -speedxmax) flake->vx = -speedxmax;
+      P("vxa: %f\n",flake->vx);
    }
 
    flake->vy += INITIALYSPEED * (drand48()-0.4)*0.1 ;
@@ -361,6 +371,7 @@ int do_UpdateSnowFlake(Snow *flake)
 
    if (!flake->fluff)
    {
+      Lock_fallen();
       // determine if non-fluffy-flake touches the fallen snow,
       // if so: make the flake inactive.
       // the bottom pixels of the snowflake are at y = NewY + (height of flake)
@@ -385,7 +396,7 @@ int do_UpdateSnowFlake(Snow *flake)
 		  for (i = istart; i < imax; i++)
 		     if (ny > fsnow->y - fsnow->acth[i] - 1)
 		     {
-			if(fsnow->acth[i] < fsnow->h/2)
+			if(fsnow->acth[i] < fsnow->desh[i])
 			   UpdateFallenSnowPartial(fsnow,nx - fsnow->x, flakew);
 			if(HandleFallenSnow(fsnow))
 			{
@@ -403,10 +414,14 @@ int do_UpdateSnowFlake(Snow *flake)
 			      fluffify(flake,0.1);
 			   }
 			   if (flake->fluff)
+			   {
+			      Unlock_fallen();
 			      return TRUE;
+			   }
 			   else
 			   {
 			      DelFlake(flake);
+			      Unlock_fallen();
 			      return FALSE;
 			   }
 			}
@@ -417,6 +432,7 @@ int do_UpdateSnowFlake(Snow *flake)
 	    }
 	 fsnow = fsnow->next;
       }
+      Unlock_fallen();
    }
 
    int x  = lrintf(flake->rx);
@@ -467,20 +483,16 @@ int do_UpdateSnowFlake(Snow *flake)
 	    if(found) break;
 	    int ybot = y+flakeh;
 	    int xbot = x+i;
-	    //int in = XRectInRegion(global.TreeRegion,xbot,ybot,1,1);
 	    cairo_rectangle_int_t grect = {xbot,ybot,1,1};
 	    cairo_region_overlap_t in = cairo_region_contains_rectangle(global.TreeRegion,&grect);
-	    //if (in != RectangleIn) // if bottom pixel not in TreeRegion, skip
 	    if (in != CAIRO_REGION_OVERLAP_IN) // if bottom pixel not in TreeRegion, skip
 	       continue;
 	    // move upwards, until pixel is not in TreeRegion
 	    int j;
 	    for (j=ybot-1; j >= y; j--)
 	    {
-	       //int in = XRectInRegion(global.TreeRegion,xbot,j,1,1); 
 	       cairo_rectangle_int_t grect = {xbot,j,1,1};
 	       cairo_region_overlap_t in = cairo_region_contains_rectangle(global.TreeRegion,&grect); 
-	       //if (in != RectangleIn)
 	       if (in != CAIRO_REGION_OVERLAP_IN)
 	       {
 		  // pixel (xbot,j) is snow-on-tree
@@ -556,6 +568,7 @@ void EraseSnowFlake1(Snow *flake)
    P("Erasesnowflake1\n");
    if(global.IsDouble)
       return;
+   P("Erasesnowflake++++++++\n");
    int x = flake->ix-1;
    int y = flake->iy-1;
    int flakew = snowPix[flake->whatFlake].width+2;
@@ -646,8 +659,7 @@ int do_show_flakecount(void *d)
 {
    if (Flags.Done)
       return FALSE;
-   if (!Flags.NoMenu)
-      ui_show_nflakes(global.FlakeCount);
+   ui_show_nflakes(global.FlakeCount);
    return TRUE;
    (void)d;
 }
@@ -669,8 +681,8 @@ void genxpmflake(char ***xpm, int w, int h)
    float w2 = 0.5*w;
    float h2 = 0.5*h;
 
-   y[0] = -w2;
-   x[0] = -h2;    // to have at least one pixel in the centre
+   y[0] = 0;
+   x[0] = 0;    // to have at least one pixel in the centre
    int n = 1;
    for (i=0; i<h; i++)
    {
@@ -865,6 +877,6 @@ int do_SwitchFlakes(void *d)
 
 void printflake(Snow *flake)
 {
-   printf("flake: %p rx: %6.0f ry: %6.0f fluff: %d freeze: %d ftr: %8.3f ft: %8.3f\n",
-	 (void *)flake,flake->rx,flake->ry,flake->fluff,flake->freeze,flake->flufftimer,flake->flufftime);
+   printf("flake: %p rx: %6.0f ry: %6.0f vx: %6.0f vy: %6.0f ws: %6.0f fluff: %d freeze: %d ftr: %8.3f ft: %8.3f\n",
+	 (void *)flake,flake->rx,flake->ry,flake->vx,flake->vy,flake->wsens,flake->fluff,flake->freeze,flake->flufftimer,flake->flufftime);
 }
