@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020,2021 Willem Vermin
+#-# 	      2019,2020,2021,2022 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 #-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #-# 
 */
+#include <pthread.h>
 #include <stdio.h>
 #include <gtk/gtk.h>
 #include <stdlib.h>
@@ -28,6 +29,7 @@
 #include "windows.h"
 #include "pixmaps.h"
 #include "utils.h"
+#include "safe_malloc.h"
 
 #define NOTACTIVE \
    (Flags.BirdsOnly || !WorkspaceActive())
@@ -94,6 +96,7 @@ void init_stars()
    P("initstars %d\n",NStars);
    // Nstars+1: we do not allocate 0 bytes
    Stars = (Skoordinaten *) realloc(Stars,(NStars+1)*sizeof(Skoordinaten));
+   REALLOC_CHECK(Stars);
    for (i=0; i<NStars; i++)
    {
       Skoordinaten *star = &Stars[i];
