@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020,2021 Willem Vermin
+#-# 	      2019,2020,2021,2022 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 #-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #-# 
  */
+#include <pthread.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <X11/Intrinsic.h>
@@ -103,6 +104,7 @@ int make_trans_window(GtkWidget *transwindow, int fullscreen, int sticky, int be
       XWindowAttributes attr;
       Display *display = XOpenDisplay(NULL);
       XGetWindowAttributes(display,DefaultRootWindow(display),&attr);
+      P("width, height %d %d\n",attr.width,attr.height);
       gtk_widget_set_size_request(GTK_WIDGET(transwindow),attr.width,attr.height);
       XCloseDisplay(display);
    }
