@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020,2021 Willem Vermin
+#-# 	      2019,2020,2021,2022 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -73,7 +73,7 @@
  *         these call backs have names like 'button_xsnow_celestials_HaloBright'
  *         the code ensures that for example Flags.HaloBright gets the value
  *         of the corresponding button.
- *       create a function settings1(), that sets all buttons in the state
+ *       create a function set_buttons1(), that sets all buttons in the state
  *         defined by the corresponding Flags. For example, if 
  *         Flags.HaloBright = 40, the corresponding GtkScale button will be set
  *         to this value.
@@ -125,6 +125,7 @@
 *      
 */
 
+#include <pthread.h>
 #include "buttons.h"
 // undef NEWLINE if one wants to examine the by cpp generated code:
 // cpp  ui.c | sed 's/NEWLINE/\n/g'
@@ -152,6 +153,7 @@
 #include "version.h"
 #include "windows.h"
 #include "xsnow.h"
+#include "safe_malloc.h"
 
 #ifndef DEBUG
 #define DEBUG
@@ -185,15 +187,17 @@ static GtkWidget     *range;
 static GtkWidget     *desktop_type;
 static GtkContainer  *birdsgrid;
 static GtkContainer  *moonbox;
+static GtkImage      *preview;
 #define nsbuffer 512
 static char sbuffer[nsbuffer];
 
-static void set_buttons(void);
+static void set_buttons1(void);
 static void set_santa_buttons(void);
 static void set_tree_buttons(void);
 static void handle_css(void);
 static void birdscb(GtkWidget *w, void *m);
 static int  below_confirm_ticker(void *);
+static int  ui_running = False;
 static void show_bct_countdown(void);
 static void yesyes(GtkWidget *w, gpointer data);
 static void nono(GtkWidget *w, gpointer data);
@@ -201,6 +205,8 @@ static void activate (GtkApplication *app);
 static void set_default_tab(int tab, int vintage);
 static void set_belowall_default();
 static void handle_theme(void);
+static void update_preview_cb (GtkFileChooser *file_chooser, gpointer data);
+static void my_gtk_label_set_text(GtkLabel *label, const gchar *str); 
 
 static int human_interaction = 1;
 GtkWidget *nflakeslabel;
@@ -218,6 +224,8 @@ void ui_ui()
 
 void handle_theme()
 {
+   if(!ui_running)
+      return;
    if (Flags.ThemeXsnow)
    {
       gtk_style_context_add_class(hauptfenstersc,"xsnow");
@@ -327,6 +335,7 @@ static struct _tree_buttons
 #define togglecode(type,name,m) NEWLINE GtkWidget *name;
 #define scalecode togglecode
 #define colorcode togglecode
+#define filecode  togglecode
 static struct _Button 
 {
    ALL_BUTTONS
@@ -342,6 +351,7 @@ static struct _Button
    NEWLINE Button.name = GTK_WIDGET(gtk_builder_get_object(builder,ID "-" #name));
 #define scalecode togglecode
 #define colorcode togglecode
+#define filecode  togglecode 
 
 static void init_buttons1()
 {
@@ -349,6 +359,7 @@ static void init_buttons1()
    ALL_BUTTONS
       P("\nend init_buttons1\n\n");
 }
+
 #include "undefall.inc"
 
 // define call backs
@@ -382,6 +393,17 @@ static void init_buttons1()
       NEWLINE    rgba2color(&color,&Flags.name); \
       NEWLINE }
 
+#define filecode(type,name,m) \
+   NEWLINE MODULE_EXPORT void buttoncb(type,name)(GtkWidget *w) \
+   NEWLINE { \
+      NEWLINE    if(!human_interaction) return; \
+      NEWLINE    gchar *filename; \
+      NEWLINE    filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(w)); \
+      NEWLINE    free(Flags.name); \
+      NEWLINE    Flags.name = strdup(filename); \
+      NEWLINE    g_free(filename); \
+      NEWLINE }
+
 ALL_BUTTONS
 #include "undefall.inc"
 
@@ -396,10 +418,15 @@ NEWLINE     if (m) { \
 #define scalecode(type,name,m) \
    NEWLINE P("range %s %s %d %d\n",#name,#type,m,Flags.name); \
 NEWLINE     gtk_range_set_value(GTK_RANGE(Button.name), m*((gdouble)Flags.name));
+//NEWLINE     if(gtk_orientable_get_orientation(GTK_ORIENTABLE(Button.name)) == GTK_ORIENTATION_HORIZONTAL) 
+//NEWLINE         gtk_widget_set_size_request(GTK_WIDGET(Button.name),100,-1);
 #define colorcode(type,name,m) \
    NEWLINE P("color %s %s %d %s\n",#name,#type,m,Flags.name); \
 NEWLINE     gdk_rgba_parse(&color,Flags.name); \
 NEWLINE        gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(Button.name),&color); 
+#define filecode(type,name,m) \
+   NEWLINE P("file %s %s %d %s\n",#name,#type,m,Flags.name); \
+NEWLINE  gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(Button.name),Flags.BackgroundFile);
 
 static void set_buttons1()
 {
@@ -419,6 +446,9 @@ static void set_buttons1()
 #define colorcode(type,name,m)  \
    NEWLINE P("%s %s\n",#name,#type); \
    NEWLINE g_signal_connect(G_OBJECT(Button.name),"color-set", G_CALLBACK(buttoncb(type,name)),NULL);
+#define filecode(type,name,m) \
+   NEWLINE P("%s %s\n",#name,#type); \
+   NEWLINE g_signal_connect(G_OBJECT(Button.name),"file-set", G_CALLBACK(buttoncb(type,name)),NULL);
 
 static void connect_signals()
 {
@@ -428,6 +458,7 @@ static void connect_signals()
 }
 #include "undefall.inc"
 
+
 static void report_tree_type(int p, gint active)
 {
    P("Tree: %d %d %s\n",p,active,Flags.TreeType);
@@ -437,6 +468,7 @@ static void report_tree_type(int p, gint active)
    if(active)
    {
       a = (int *)realloc(a,sizeof(*a)*(n+1));
+      REALLOC_CHECK(a);
       a[n] = p;
       n++;
    }
@@ -617,17 +649,20 @@ MODULE_EXPORT void button_below(GtkWidget *w)
     */
    if(!human_interaction) return;
    gint active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
-   P("button_below: %d\n",Flags.BelowAll);
-   if(active)
-      Flags.BelowAll = 1;
-   else
+   P("button_below: %d %d\n",Flags.BelowAllForce, Flags.BelowAll);
+   if (!Flags.BelowAllForce)
    {
-      Flags.BelowAll = 0;
-      bct_countdown  = 9;
-      show_bct_countdown();
-      gtk_widget_hide(Button.BelowAll);
-      gtk_widget_show(Button.BelowConfirm);
-      bct_id = add_to_mainloop(PRIORITY_DEFAULT,1.0,below_confirm_ticker);
+      if(active)
+	 Flags.BelowAll = 1;
+      else
+      {
+	 Flags.BelowAll = 0;
+	 bct_countdown  = 9;
+	 show_bct_countdown();
+	 gtk_widget_hide(Button.BelowAll);
+	 gtk_widget_show(Button.BelowConfirm);
+	 bct_id = add_to_mainloop(PRIORITY_DEFAULT,time_below_confirm,below_confirm_ticker);
+      }
    }
 }
 MODULE_EXPORT void button_below_confirm()
@@ -642,7 +677,7 @@ static void init_general_buttons()
    g_signal_connect(Button.BelowAll, "toggled", G_CALLBACK (button_below), NULL);
    g_signal_connect(Button.BelowConfirm, "toggled", G_CALLBACK(button_below_confirm), NULL);
 
-   gtk_label_set_text(GTK_LABEL(gtk_builder_get_object(builder,"id-version")),"xsnow version " VERSION);
+   my_gtk_label_set_text(GTK_LABEL(gtk_builder_get_object(builder,"id-version")),"xsnow version " VERSION);
 
    gtk_widget_hide(Button.BelowConfirm);
 }
@@ -653,7 +688,6 @@ void show_bct_countdown()
 {
    sprintf(sbuffer,"Click to\nconfirm %d",bct_countdown);
    gtk_button_set_label(GTK_BUTTON(Button.BelowConfirm),sbuffer);
-
 }
 
 int below_confirm_ticker(void *d)
@@ -726,18 +760,23 @@ void button_vintage_snow()
 
 void ui_set_birds_header(const char *text)
 {
+   if(!ui_running)
+      return;
    GtkWidget *birds_header = GTK_WIDGET(gtk_builder_get_object(builder,"birds-header")); 
-   gtk_label_set_text(GTK_LABEL(birds_header),text);
+   my_gtk_label_set_text(GTK_LABEL(birds_header),text);
 }
 
 void ui_set_celestials_header(const char *text)
 {
+   if(!ui_running)
+      return;
    GtkWidget *celestials_header = GTK_WIDGET(gtk_builder_get_object(builder,"celestials-header")); 
    char *a = strdup(gtk_label_get_text(GTK_LABEL(celestials_header)));
    a = (char *) realloc(a,strlen(a)+2+strlen(text));
+   REALLOC_CHECK(a);
    strcat(a,"\n");
    strcat(a,text);
-   gtk_label_set_text(GTK_LABEL(celestials_header),a);
+   my_gtk_label_set_text(GTK_LABEL(celestials_header),a);
    free(a);
 }
 
@@ -766,10 +805,17 @@ MODULE_EXPORT void button_wind_activate()
    Flags.WindNow = 1;
 }
 
+MODULE_EXPORT void button_xscreensaver_activate()
+{
+   int rc = system("xscreensaver-command -activate");
+   (void) rc;
+}
+
 void set_default_tab(int tab, int vintage)
 {
    int h = human_interaction;
    human_interaction = 0;
+   char *background = strdup(Flags.BackgroundFile); // don't want to clear backgroundfile
    if(vintage)
    {
 #define togglecode(type,name,m) \
@@ -778,6 +824,7 @@ void set_default_tab(int tab, int vintage)
 #define colorcode(type,name,m) \
       NEWLINE    if (type == tab) \
       NEWLINE       { free(Flags.name); Flags.name = strdup(VINTAGE(name)); } 
+#define filecode colorcode
 
       ALL_BUTTONS;
 #include "undefall.inc"
@@ -796,6 +843,8 @@ void set_default_tab(int tab, int vintage)
 	    break;
 	 case xsnow_settings:
 	    set_belowall_default();
+	    free(Flags.BackgroundFile);
+	    Flags.BackgroundFile = strdup(background);
 	    break;
       }
    }
@@ -806,6 +855,7 @@ void set_default_tab(int tab, int vintage)
 #define colorcode(type,name,m) \
       NEWLINE    if (type == tab) \
       NEWLINE       { free(Flags.name); Flags.name = strdup(DEFAULT(name)); } 
+#define filecode colorcode
    {
       ALL_BUTTONS;
 #include "undefall.inc"
@@ -824,11 +874,14 @@ void set_default_tab(int tab, int vintage)
 	    break;
 	 case xsnow_settings:
 	    set_belowall_default();
+	    free(Flags.BackgroundFile);
+	    Flags.BackgroundFile = strdup(background);
 	    break;
       }
    }
    set_buttons();
    human_interaction = h;
+   free(background);
 }
 
    MODULE_EXPORT
@@ -854,7 +907,7 @@ static void init_buttons()
    nflakeslabel = GTK_WIDGET(gtk_builder_get_object(builder,"nflakes"));
 }
 
-static void set_buttons()
+void set_buttons()
 {
    human_interaction = 0;
    set_buttons1();
@@ -888,37 +941,76 @@ MODULE_EXPORT void button_all_vintage()
 
 void ui_show_nflakes(int n)
 {
+   if(!ui_running)
+      return;
    snprintf(sbuffer,nsbuffer,"%6d",n);
-   gtk_label_set_text(GTK_LABEL(nflakeslabel),sbuffer);
+   my_gtk_label_set_text(GTK_LABEL(nflakeslabel),sbuffer);
 }
 
 void ui_show_range_etc()
 {
+   if(!ui_running)
+      return;
    snprintf(sbuffer,nsbuffer,"Range: %d\n",(int)birds_get_range());
-   gtk_label_set_text(GTK_LABEL(range),sbuffer);
+   my_gtk_label_set_text(GTK_LABEL(range),sbuffer);
    snprintf(sbuffer,nsbuffer,"Mean dist: %d\n",(int)birds_get_mean_dist());
-   gtk_label_set_text(GTK_LABEL(mean_distance),sbuffer);
+   my_gtk_label_set_text(GTK_LABEL(mean_distance),sbuffer);
 }
 
 void ui_show_desktop_type(const char *s)
 {
+   if(!ui_running)
+      return;
    snprintf(sbuffer,nsbuffer,"Desktop type: %s",s);
-   gtk_label_set_text(GTK_LABEL(desktop_type),sbuffer);
+   my_gtk_label_set_text(GTK_LABEL(desktop_type),sbuffer);
 }
 
 void ui_set_sticky(int x)
 {
+   if(!ui_running)
+      return;
    if (x)
       gtk_window_stick(GTK_WINDOW(hauptfenster));
    else
       gtk_window_unstick(GTK_WINDOW(hauptfenster));
 }
 
+// https://docs.gtk.org/gtk3/iface.FileChooser.html
+void update_preview_cb (GtkFileChooser *file_chooser, gpointer data)
+{
+   GtkWidget *preview;
+   char *filename;
+   GdkPixbuf *pixbuf;
+   gboolean have_preview;
+
+   preview = GTK_WIDGET (data);
+   filename = gtk_file_chooser_get_preview_filename (file_chooser);
+   if(!IsReadableFile(filename))
+   {
+      g_free(filename);
+      return;
+   }
+
+   int w = global.SnowWinWidth/10;
+
+   //pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 128, 128, NULL);
+   pixbuf = gdk_pixbuf_new_from_file_at_size (filename, w, w, NULL);
+   have_preview = (pixbuf != NULL);
+   g_free (filename);
+
+   gtk_image_set_from_pixbuf (GTK_IMAGE (preview), pixbuf);
+   if (pixbuf)
+      g_object_unref (pixbuf);
+
+   gtk_file_chooser_set_use_preview_label(file_chooser, FALSE);
+   gtk_file_chooser_set_preview_widget_active (file_chooser, have_preview);
+}
+
 void ui()
 {
-
+   ui_running = True;
    builder = gtk_builder_new_from_string (xsnow_xml, -1);
-   gtk_builder_connect_signals (builder, builder);
+   gtk_builder_connect_signals  (builder, NULL);
    hauptfenster  = GTK_WIDGET   (gtk_builder_get_object(builder, "hauptfenster"));
    mean_distance = GTK_WIDGET   (gtk_builder_get_object(builder, "birds-mean-distance"));
    range         = GTK_WIDGET   (gtk_builder_get_object(builder, "birds-range"));
@@ -937,9 +1029,14 @@ void ui()
    connect_signals();
    init_pixmaps();
    set_buttons();
+   preview = GTK_IMAGE(gtk_image_new());
+   gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER(Button.BackgroundFile), GTK_WIDGET(preview));
+   g_signal_connect (GTK_FILE_CHOOSER(Button.BackgroundFile), "update-preview",
+	 G_CALLBACK (update_preview_cb), preview);
 
    if (Flags.HideMenu)
       gtk_window_iconify(GTK_WINDOW(hauptfenster));
+
 }
 
 // Set the style provider for the widgets
@@ -972,7 +1069,7 @@ void handle_css()
       //".xsnow button.color {padding: 4px; }"
       //".xsnow headerbar stackswitcher button.radio label       { color: #065522;  }"   
       //".xsnow headerbar stackswitcher button.radio        { box-shadow: 0px 0px; border-top-width: 0px;  }"   
-      
+
       // These are not colors, but nevertheless I think we should do this always:
       "scale              { padding:       1em;                    }"   // padding in slider buttons
       "button.radio       { min-width:     10px;                   }"   // make window as narrow as possible
@@ -1014,6 +1111,8 @@ void handle_css()
 
 void ui_background(int m)
 {
+   if(!ui_running)
+      return;
    if(m)
       gtk_style_context_add_class(hauptfenstersc,"busy");
    else
@@ -1028,7 +1127,8 @@ void ui_background(int m)
 void ui_gray_erase(int m)
 {
    gtk_widget_set_sensitive(Button.BelowAll,                      m);
-   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(Button.BelowAll),1);
+   if(!Flags.BelowAllForce)
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(Button.BelowAll),1);
 }
 
 
@@ -1046,6 +1146,8 @@ void birdscb(GtkWidget *w, void *m)
 
 void ui_gray_birds(int m)
 {
+   if(!ui_running)
+      return;
    gtk_container_foreach(birdsgrid, birdscb, &m);
    gtk_container_foreach(moonbox, birdscb, &m);
 }
@@ -1089,7 +1191,15 @@ static int RC;
 int ui_run_nomenu()
 {
    GtkApplication *app;
-   app = gtk_application_new ("nl.ratrabbit.example", G_APPLICATION_FLAGS_NONE);
+   // see https://fuchsia.googlesource.com/third_party/glib/+/refs/heads/upstream/main/NEWS
+#define MY_GLIB_VERSION (100000000*GLIB_MAJOR_VERSION + 10000*GLIB_MINOR_VERSION + GLIB_MICRO_VERSION)
+#if MY_GLIB_VERSION >= 200730003
+#define XXFLAGS G_APPLICATION_DEFAULT_FLAGS
+#else
+#define XXFLAGS G_APPLICATION_FLAGS_NONE
+#endif
+
+   app = gtk_application_new ("nl.ratrabbit.example", XXFLAGS);
    g_signal_connect (app, "activate", G_CALLBACK (activate), NULL);
    g_application_run (G_APPLICATION (app), 0, NULL);
    g_object_unref (app);
@@ -1103,7 +1213,6 @@ static void activate (GtkApplication *app)
    GtkWidget *button;
    GtkWidget *label;
 
-
    /* create a new window, and set its title */
    window = gtk_application_window_new (app);
    gtk_window_set_position        (GTK_WINDOW(window), GTK_WIN_POS_CENTER);
@@ -1191,3 +1300,9 @@ void ui_error_x11()
    gtk_widget_show_all (errorfenster);
    gtk_main();
 }
+
+void my_gtk_label_set_text(GtkLabel *label, const gchar *str)
+{
+   if(ui_running)
+      gtk_label_set_text(label, str);
+}
