@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020,2021 Willem Vermin
+#-# 	      2019,2020,2021,2022 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -19,11 +19,13 @@
 #-# 
 */
 
+#include <pthread.h>
 #include <stdio.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <X11/Intrinsic.h>
 #include <ctype.h>
+//#include <byteswap.h>
 #include "debug.h"
 #include "windows.h"
 #include "flags.h"
@@ -33,14 +35,15 @@
 #include "fallensnow.h"
 #include "transwindow.h"
 #include "dsimple.h"
+#include "safe_malloc.h"
 
 #include "vroot.h"
-static int    do_wupdate(void *);
 static int    do_sendevent(void *);
 static long   TransWorkSpace = -SOMENUMBER;  // workspace on which transparent window is placed
 
 static WinInfo      *Windows = NULL;
 static int          NWindows;
+static int          do_wupdate(void *);
 
 void windows_ui()
 {
@@ -63,7 +66,7 @@ void windows_init()
    if (global.Desktop)
       add_to_mainloop(PRIORITY_DEFAULT, time_wupdate, do_wupdate);
    if (!global.IsDouble)
-      add_to_mainloop(PRIORITY_DEFAULT, 0.5, do_sendevent);
+      add_to_mainloop(PRIORITY_DEFAULT, time_sendevent, do_sendevent);
 }
 
 int WorkspaceActive()
@@ -93,6 +96,12 @@ int do_sendevent(void *dummy)
    (void)dummy;
 }
 
+void UpdateWindows()
+{
+   global.WindowsChanged = 1;
+   do_wupdate((void*)0);
+}
+
 int do_wupdate(void *dummy)
 {
    P("do_wupdate %d %d\n",counter++,global.WindowsChanged);
@@ -101,8 +110,24 @@ int do_wupdate(void *dummy)
 
    if(Flags.NoKeepSWin) return TRUE;
 
-   if (!global.WindowsChanged)
+   static int lockcounter = 0;
+   if (Lock_fallen_n(3,&lockcounter))
+   {
+      P("lock counter: %d\n",lockcounter);
       return TRUE;
+   }
+   P("do_wupdate running %d\n",lockcounter);
+
+   // once in a while, we force updating windows
+   static int wcounter = 0;
+   wcounter++;
+   if (wcounter > 9)
+   {
+      global.WindowsChanged = 1;
+      wcounter = 0;
+   }
+   if (!global.WindowsChanged)
+      goto end;
 
    global.WindowsChanged = 0;
 
@@ -114,7 +139,7 @@ int do_wupdate(void *dummy)
    {
       I("Cannot get current workspace\n");
       Flags.Done = 1;
-      return TRUE;
+      goto end;
    }
 
 
@@ -138,7 +163,7 @@ int do_wupdate(void *dummy)
    {
       I("Cannot get windows\n");
       Flags.Done = 1;
-      return TRUE;
+      goto end;
    }
 
    //P("%d:\n",counter++);printwindows(display,Windows,NWindows);
@@ -175,16 +200,27 @@ int do_wupdate(void *dummy)
       }
 
    UpdateFallenSnowRegions();
+
+end:
+   Unlock_fallen();
    return TRUE;
    (void)dummy;
 }
 
+void UpdateFallenSnowRegionsWithLock()
+{
+   Lock_fallen();
+   UpdateFallenSnowRegions();
+   Unlock_fallen();
+}
+
 // Have a look at the windows we are snowing on
 // Also update of fallensnow area's
 void UpdateFallenSnowRegions()
 {
+   // threads: locking by caller
    WinInfo *w;
-   FallenSnow *f;
+   FallenSnow *fsnow;
    int i;
    // add fallensnow regions:
    w = Windows;
@@ -192,28 +228,27 @@ void UpdateFallenSnowRegions()
    {
       //P("%d %#lx\n",i,w->id);
       {
-	 f = FindFallen(global.FsnowFirst,w->id);
+	 fsnow = FindFallen(global.FsnowFirst,w->id);
 	 P("%#lx %d\n",w->id,w->dock);
-	 if(f)
+	 if(fsnow)
 	 {
-	    f->win = *w;   // update window properties
-	    if ((!f->win.sticky) && f->win.ws != global.CWorkSpace)
+	    fsnow->win = *w;   // update window properties
+	    if ((!fsnow->win.sticky) && fsnow->win.ws != global.CWorkSpace)
 	    {
 	       P("CleanFallenArea\n");
-	       CleanFallenArea(f,0,f->w);
+	       CleanFallenArea(fsnow,0,fsnow->w);
 	    }
 	 }
-	 if (!f)
+	 if (!fsnow)
 	 {
 	    // window found in Windows, nut not in list of fallensnow,
 	    // add it, but not if we are snowing or birding in this window (Desktop for example)
 	    // and also not if this window has y <= 0
 	    // and also not if this window is a "dock"
 	    P("               %#lx %d\n",w->id,w->dock);
-	    // if (w->id != SnowWin_a && w->id != SnowWinb && w->y > 0 && !(w->dock)) // let op
 	    if (w->id != global.SnowWin && w->y > 0 && !(w->dock)) 
 	    {
-	       if((int)(w->w) == global.SnowWinWidth && w->x == 0 && w->y <100) //maybe a transparent xpenguins window?
+	       if(((int)(w->w) == global.SnowWinWidth && w->x == 0 && w->y <100)) //maybe a transparent xpenguins window?
 	       {
 		  P("skipping: %d %#lx %d %d %d\n",global.counter++, w->id, w->w, w->x, w->y);
 	       }
@@ -230,40 +265,46 @@ void UpdateFallenSnowRegions()
       w++;
    }
    // remove fallensnow regions
-   f = global.FsnowFirst; 
+   fsnow = global.FsnowFirst; 
    int nf = 0; 
-   while(f) 
+   while(fsnow) 
    { 
       nf++; 
-      f = f->next; 
+      fsnow = fsnow->next; 
    }
    // nf+1: prevent allocation of zero bytes
    long int *toremove = (long int *)malloc(sizeof(*toremove)*(nf+1));
    int ntoremove = 0;
-   f = global.FsnowFirst;
-   while(f)
+   fsnow = global.FsnowFirst;
+   while(fsnow)
    {
-      if (f->win.id != 0)  // f->id=0: this is the snow at the bottom
+      if (fsnow->win.id != 0)  // fsnow->id=0: this is the snow at the bottom
       {
-	 w = FindWindow(Windows,NWindows,f->win.id);
-	 if(!w)   // this window is gone
+	 w = FindWindow(Windows,NWindows,fsnow->win.id);
+	 if(
+	       !w                                                       // this window is gone
+	       || ( w->w > 0.8*global.SnowWinWidth 
+		  && w->ya < Flags.IgnoreTop)                           // too wide&too close to top   
+	       || ( w->w > 0.8*global.SnowWinWidth 
+		  && global.SnowWinHeight - w->ya < Flags.IgnoreBottom) // too wide&too close to bottom
+	   )
 	 {
-	    GenerateFlakesFromFallen(f,0,f->w,-10.0);
-	    toremove[ntoremove++] = f->win.id;
+	    GenerateFlakesFromFallen(fsnow,0,fsnow->w,-10.0);
+	    toremove[ntoremove++] = fsnow->win.id;
 	 }
 
-	 // test if f->win.id is hidden. If so: clear the area and notify in f
+	 // test if fsnow->win.id is hidden. If so: clear the area and notify in fsnow
 	 // we have to test that here, because the hidden status of the window
 	 // can change
-	 P("%#lx hidden:%d\n",f->win.id,f->win.hidden);
-	 if (f->win.hidden)
+	 P("%#lx hidden:%d\n",fsnow->win.id,fsnow->win.hidden);
+	 if (fsnow->win.hidden)
 	 {
-	    P("%#lx is hidden %d\n",f->win.id, counter++);
-	    CleanFallenArea(f,0,f->w);
+	    P("%#lx is hidden %d\n",fsnow->win.id, counter++);
+	    CleanFallenArea(fsnow,0,fsnow->w);
 	    P("CleanFallenArea\n");
 	 }
       }
-      f = f->next;
+      fsnow = fsnow->next;
    }
 
    // test if window has been moved or resized
@@ -272,24 +313,24 @@ void UpdateFallenSnowRegions()
    w = Windows;
    for(i=0; i<NWindows; i++)
    {
-      f = FindFallen(global.FsnowFirst,w->id);
-      if (f)
+      fsnow = FindFallen(global.FsnowFirst,w->id);
+      if (fsnow)
       {
-	 if ((unsigned int)f->w == w->w+Flags.OffsetW) // width has not changed
+	 if ((unsigned int)fsnow->w == w->w+Flags.OffsetW) // width has not changed
 	 {
-	    if (f->x != w->x + Flags.OffsetX || f->y != w->y + Flags.OffsetY)
+	    if (fsnow->x != w->x + Flags.OffsetX || fsnow->y != w->y + Flags.OffsetY)
 	    {
-	       CleanFallenArea(f,0,f->w);
+	       CleanFallenArea(fsnow,0,fsnow->w);
 	       P("CleanFallenArea\n");
-	       f->x = w->x + Flags.OffsetX;
-	       f->y = w->y + Flags.OffsetY;
-	       DrawFallen(f);
+	       fsnow->x = w->x + Flags.OffsetX;
+	       fsnow->y = w->y + Flags.OffsetY;
+	       //DrawFallen(fsnow);
 	       XFlush(global.display);
 	    }
 	 }
 	 else
 	 {
-	    toremove[ntoremove++] = f->win.id;
+	    toremove[ntoremove++] = fsnow->win.id;
 	 }
       }
       w++;
@@ -580,6 +621,8 @@ void InitDisplayDimensions()
 
 void DisplayDimensions()
 {
+   P("Displaydimensions\n");
+   Lock_fallen();
    unsigned int w,h,b,d;
    int x,y,xr,yr;
    Window root,child_return;
@@ -605,8 +648,8 @@ void DisplayDimensions()
    }
    else
    {
-      global.SnowWinHeight      = h + Flags.OffsetS;
       global.SnowWinWidth       = w;
+      global.SnowWinHeight      = h + Flags.OffsetS;
    }
    P("DisplayDimensions: SnowWinX: %d Y:%d W:%d H:%d\n",global.SnowWinX,global.SnowWinY,global.SnowWinWidth,global.SnowWinHeight);
 
@@ -618,5 +661,91 @@ void DisplayDimensions()
    SetMaxScreenSnowDepth();
    if(!global.IsDouble)
       ClearScreen();
+   Unlock_fallen();
 }
 
+void SetBackground()
+{
+   char *f = Flags.BackgroundFile;
+   if (!IsReadableFile(f))
+      return;
+
+   printf("Setting background from %s\n",f);
+
+   int w = global.SnowWinWidth;
+   int h = global.SnowWinHeight;
+   Display *display = global.display;
+   Window window = global.SnowWin;
+   int screen_num = DefaultScreen(display);
+   int depth = DefaultDepth(display, screen_num);
+
+   GdkPixbuf *pixbuf;
+   pixbuf = gdk_pixbuf_new_from_file_at_scale(f,w,h,FALSE,NULL);
+   if (!pixbuf)
+      return;
+   int n_channels = gdk_pixbuf_get_n_channels(pixbuf);
+
+   guchar *pixels = gdk_pixbuf_get_pixels(pixbuf);
+   P("pad: %d %d\n",XBitmapPad(display),depth);
+
+   unsigned char *pixels1 = (unsigned char*)malloc(w*h*4*sizeof(unsigned char));
+   // https://gnome.pages.gitlab.gnome.org/gdk-pixbuf/gdk-pixbuf/class.Pixbuf.html
+   //
+
+   int rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+   P("rowstride: %d\n",rowstride);
+   int i,j;
+   int k = 0;
+   if(is_little_endian())
+      for (i=0; i<h; i++)
+	 for(j=0; j<w; j++)
+	 {
+	    guchar *p = &pixels[i*rowstride +j*n_channels];
+	    pixels1[k++] = p[2];
+	    pixels1[k++] = p[1];
+	    pixels1[k++] = p[0];
+	    pixels1[k++] = 0xff;
+	 }
+   else
+   {
+      I("Big endian system, swapping bytes in background.\n");
+      I("Let me know if this is not OK.\n");
+      for (i=0; i<h; i++)
+	 for(j=0; j<w; j++)
+	 {
+	    guchar *p = &pixels[i*rowstride +j*n_channels];
+	    pixels1[k++] = 0xff;
+	    pixels1[k++] = p[0];
+	    pixels1[k++] = p[1];
+	    pixels1[k++] = p[2];
+	 }
+   }
+
+   XImage *ximage;
+   ximage = XCreateImage(display, 
+	 DefaultVisual(display, screen_num),
+	 depth,
+	 ZPixmap,
+	 0,
+	 (char*)pixels1,
+	 w,
+	 h,
+	 XBitmapPad(display),
+	 0
+	 );
+   XInitImage(ximage);
+   Pixmap pixmap;
+   pixmap = XCreatePixmap(display,window,w,h,DefaultDepth(display,screen_num));
+
+   GC gc;
+   gc = XCreateGC(display,pixmap,0,0);
+   XPutImage(display,pixmap,gc,ximage,0,0,0,0,w,h);
+
+   P("setwindowbackground\n");
+   XSetWindowBackgroundPixmap(display,window,pixmap);
+   g_object_unref(pixbuf);
+   XFreePixmap(display, pixmap);
+   XDestroyImage(ximage);
+   //free(pixels1);  //This is already freed by XDestroyImage
+   return;
+}
