@@ -2,7 +2,7 @@
 #-# 
 #-# xsnow: let it snow on your desktop
 #-# Copyright (C) 1984,1988,1990,1993-1995,2000-2001 Rick Jansen
-#-# 	      2019,2020,2021 Willem Vermin
+#-# 	      2019,2020,2021,2022 Willem Vermin
 #-# 
 #-# This program is free software: you can redistribute it and/or modify
 #-# it under the terms of the GNU General Public License as published by
@@ -34,4 +34,8 @@ extern void         setabove(GtkWindow *w);
 extern void         DisplayDimensions(void);
 extern Window       XWinInfo(char **name);
 extern void         UpdateFallenSnowRegions(void);
+extern void         UpdateFallenSnowRegionsWithLock(void);
+extern void         UpdateWindows(void);
+extern void         SetBackground(void);
+
 
